package com.chelvc.cloud.vehicle.api.param;

import com.chelvc.cloud.vehicle.api.constant.CouponType;
import com.chelvc.cloud.vehicle.api.constant.UserCouponStatus;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;
import lombok.experimental.SuperBuilder;

import javax.validation.constraints.Max;
import javax.validation.constraints.Min;
import javax.validation.constraints.NotNull;
import java.io.Serializable;
import java.math.BigDecimal;
import java.util.List;

/**
 * 用户优惠券查询参数
 *
 * @author qizai
 * @date 2023/9/8
 */
@Data
@SuperBuilder
@NoArgsConstructor
@AllArgsConstructor
public class UserCouponQueryParam implements Serializable {

    /**
     * 偏移用户优惠券ID
     */
    @Min(value = 1, message = "偏移用户优惠券ID不能小于1")
    private Long offset;

    /**
     * 查询数量
     */
    @NotNull(message = "查询数量不能为空")
    @Min(value = 1, message = "查询数量不能小于1")
    @Max(value = 20, message = "查询数量不能大于20")
    private Integer size;

    /**
     * 优惠券类型
     */
    @NotNull(message = "优惠券类型不能为空")
    private CouponType type;

    /**
     * 用户优惠券状态
     */
    @NotNull(message = "用户优惠券状态不能为空")
    private UserCouponStatus status;

    /**
     * 总金额（元）
     */
    private BigDecimal totalAmount;

    /**
     * 商家ID列表
     */
    private List<String> merchantIds;

    /**
     * 商家ID，如有多个','分割
     */
    private String merchantId;
}
