package com.chelvc.cloud.vehicle.api.service;

import com.chelvc.cloud.vehicle.api.dto.CouponDTO;
import com.chelvc.cloud.vehicle.api.param.CouponModifyParam;
import com.chelvc.cloud.vehicle.api.param.CouponPagingParam;
import com.chelvc.framework.common.model.Pagination;

/**
 * 优惠券业务接口
 *
 * @author qizai
 * @date 2023/9/8
 */
public interface CouponService {
    /**
     * 新增优惠券
     *
     * @param param 新增参数
     * @return 优惠券主键
     */
    Long addCoupon(CouponModifyParam param);

    /**
     * 修改优惠券
     *
     * @param id    优惠券主键
     * @param param 修改参数
     */
    void updateCoupon(Long id, CouponModifyParam param);

    /**
     * 删除商品优惠券
     *
     * @param id    商品优惠券主键
     */
    void deleteCoupon(Long id);

    /**
     * 获取优惠券信息
     *
     * @param id 优惠券主键
     * @return 优惠券信息
     */
    CouponDTO getCoupon(Long id);

    /**
     * 查询优惠券信息
     *
     * @param param 查询参数
     * @return 优惠券分页信息
     */
    Pagination<CouponDTO> getCouponPaging(CouponPagingParam param);

    /**
     * 查询优惠券信息
     *
     * @param merchantId 商家ID
     * @param param 分页参数
     * @return 优惠券分页信息
     */
    Pagination<CouponDTO> getMerchantCouponPaging(Long merchantId, CouponPagingParam param);
}
