package com.chelvc.cloud.vehicle.api.service;

import com.chelvc.cloud.vehicle.api.constant.ExamineType;
import com.chelvc.cloud.vehicle.api.dto.DynamicContentDTO;
import com.chelvc.cloud.vehicle.api.param.AddDynamicContentParam;
import com.chelvc.cloud.vehicle.api.param.QueryAdminDynamicParam;
import com.chelvc.cloud.vehicle.api.param.QueryDynamicContentParam;
import com.chelvc.framework.common.model.Pagination;

import java.util.Map;

/**
 * 动态信息Service接口
 *
 * @USER: igl
 * @date: 2023/8/8 15:13
 */
public interface DynamicContentService {

  /**
   * 查询动态信息
   */
  DynamicContentDTO queryById(Long id, Long userId);

  /**
   * 查询动态信息列表
   */
  Pagination<DynamicContentDTO> queryPageList(QueryDynamicContentParam param, Integer pageNum,
                                              Integer pageSize, Long userId);

  /**
   * 查询动态信息列表
   */
  Pagination<DynamicContentDTO> queryAdminPageList(QueryAdminDynamicParam param, Long pageNum,
                                                Long pageSize);

  /**
   * 查询动态信息列表
   */
  Pagination<DynamicContentDTO> queryUserById(Long selfUserId, Long userId, Integer pageNum, Integer pageSize);

  /**
   * 修改动态信息
   */
  void insertByBo(AddDynamicContentParam dto, Long userId);

  /**
   * 校验并批量删除动态信息信息
   */
  void deleteByIds(Long id, Long userId);

  /**
   * 添加评论数
   *
   * @param dynamicId
   */
  void addComment(Long dynamicId);

  /**
   * 减少评论数
   *
   * @param dynamicId
   */
  void reduceComment(Long dynamicId, int num);

  /**
   * 给评论添加点赞/踩
   *
   * @param dynamicStatus
   * @param contentId
   */
  void addNum(Integer dynamicStatus, Long contentId);

  /**
   * 取消评论点赞/踩
   *
   * @param dynamicStatus
   * @param contentId
   */
  void reduceNum(Integer dynamicStatus, Long contentId);

  /**
   * 屏蔽/恢复某条动态信息
   *
   * @param id 主键
   */
  void shieldById(Long id, Integer type);

  /**
   * 屏蔽/恢复某个用户所有动态信息
   *
   * @param userId 主键
   * @param type   状态 0-正常；1-平台屏蔽
   */
  void shieldByUserId(Long userId, Integer type);

  /**
   * 动态手动审核
   *
   * @param id
   * @param examine
   * @author igl
   * @date 2023/8/17 17:25
   */
  void examine(Long id, ExamineType examine);

  /**
   * 自动审核
   *
   * @param id 动态标识
   * @param platformStatus 平台状态
   * @param examineResult 审核结果
   * @author igl
   * @date 2023/9/12 17:25
   */
  void autoAudit(Long id, int platformStatus, int examineResult);

  /**
   * 视频审核回调
   *
   * @param param
   * @author igl
   * @date 2023/9/12 16:55
   */
  void videoAuditCallback(Map<String, Object> param);
}
