package com.chelvc.cloud.vehicle.api.service;

import com.chelvc.cloud.vehicle.api.dto.ConfirmOrderResultDTO;
import com.chelvc.cloud.vehicle.api.dto.GoodsDTO;
import com.chelvc.cloud.vehicle.api.dto.OmsOrderDTO;
import com.chelvc.cloud.vehicle.api.dto.OmsOrderDetailDTO;
import com.chelvc.cloud.vehicle.api.param.GoodsPagingParam;
import com.chelvc.cloud.vehicle.api.param.OmsOrderModifyParam;
import com.chelvc.cloud.vehicle.api.param.OrderPagingParam;
import com.chelvc.framework.common.model.Pagination;

import java.util.List;
import java.util.Map;

/**
 * 订单Service接口
 * 
 * @author liude
 * @date 2023-11-08
 */
public interface IOmsOrderService
{

    /**
     * 根据用户购物车信息生成确认单信息
     * @param cartIds
     */
    ConfirmOrderResultDTO generateConfirmOrder(OmsOrderModifyParam orderParam);

    /**
     * 根据提交信息生成订单
     */
    Map<String, Object> generateOrder(OmsOrderModifyParam orderParam);

    /**
     * 支付成功后的回调
     */
    Integer paySuccess(Long orderId, Integer payType);

    /**
     * 自动取消超时订单
     */
    Integer cancelTimeOutOrder();

    /**
     * 取消单个超时订单
     */
    void cancelOrder(Long orderId);

    /**
     * 发送延迟消息取消订单
     */
    void sendDelayMessageCancelOrder(Long orderId);

    /**
     * 确认收货
     */
    void confirmReceiveOrder(Long orderId);

    /**
     * 查询订单
     *
     * @param param 查询参数
     * @return 订单分页信息
     */
    Pagination<OmsOrderDTO> getOrderPaging(OrderPagingParam param);

    /**
     * 分页查询订单
     *
     * @param param 查询参数
     * @return 订单分页信息
     */
    Pagination<OmsOrderDTO> getOmsOrderPaging(OrderPagingParam param);

    /**
     * 根据订单ID获取订单详情
     */
    OmsOrderDetailDTO detail(Long orderId);

    /**
     * 用户根据订单ID删除订单
     */
    void deleteOrder(Long orderId);
}
