package com.chelvc.cloud.vehicle.api.service;


import com.chelvc.cloud.vehicle.api.dto.UserCouponDTO;
import com.chelvc.cloud.vehicle.api.param.UserCouponModifyParam;
import com.chelvc.cloud.vehicle.api.param.UserCouponPagingParam;
import com.chelvc.cloud.vehicle.api.param.UserCouponQueryParam;
import com.chelvc.framework.common.model.Pagination;

import java.util.List;

/**
 * 优惠券领取记录业务接口
 *
 * @author 七仔
 * @date 2023/3/29 21:59
 */
public interface UserCouponService {
    /**
     * 查询用户优惠券
     *
     * @param param 查询参数
     * @return 用户优惠券信息列表
     */
    List<UserCouponDTO> listUserCoupons(UserCouponQueryParam param);

    /**
     * 获取当前用户对于当前商品可使用的优惠券列表
     *
     * @param param 查询参数
     * @return 用户可用优惠券信息列表
     */
    List<UserCouponDTO> listUserCanUseCoupons(UserCouponQueryParam param);

    /**
     * 新增优惠券领取记录
     *
     * @param param 新增参数
     * @return 优惠券领取记录主键
     */
    Long addUserCoupon(UserCouponModifyParam param);

    /**
     * 修改优惠券领取记录
     *
     * @param id    优惠券领取记录主键
     * @param param 修改参数
     */
    void updateUserCoupon(Long id, UserCouponModifyParam param);

    /**
     * 获取优惠券领取记录
     *
     * @param id 优惠券领取记录主键
     * @return 优惠券领取记录
     */
    UserCouponDTO getUserCoupon(Long id);

    /**
     * 查询优惠券领取记录
     *
     * @param param 查询参数
     * @return 优惠券领取记录分页信息
     */
    Pagination<UserCouponDTO> getUserCouponPaging(UserCouponPagingParam param);

    /**
     * 领取优惠券
     *
     * @param couponId 优惠券ID
     * @return 优惠券领取记录主键
     */
    Long claimCoupon(Long couponId);
}
