package com.chelvc.cloud.vehicle.server.copier;

import com.chelvc.cloud.vehicle.api.dto.CommissionConfigDTO;
import com.chelvc.cloud.vehicle.api.dto.IncomeRecordDTO;
import com.chelvc.cloud.vehicle.api.param.CommissionConfigModifyParam;
import com.chelvc.cloud.vehicle.api.param.IncomeRecordModifyParam;
import com.chelvc.cloud.vehicle.server.entity.CommissionConfig;
import com.chelvc.cloud.vehicle.server.entity.IncomeRecord;
import org.mapstruct.Builder;
import org.mapstruct.Mapper;
import org.mapstruct.MappingTarget;
import org.mapstruct.factory.Mappers;

import java.util.Collection;
import java.util.List;

/**
 * 收益记录拷贝接口
 *
 * @author liude
 * @date 2023/12/16
 */
@Mapper(builder = @Builder(disableBuilder = true))
public interface IncomeRecordCopier {
    /**
     * 对象拷贝接口实例
     */
    IncomeRecordCopier INSTANCE = Mappers.getMapper(IncomeRecordCopier.class);

    /**
     * 收益记录信息拷贝
     *
     * @param incomeRecord 收益记录信息
     * @return 收益记录信息
     */
    IncomeRecordDTO copying(IncomeRecord incomeRecord);

    /**
     * 收益记录信息拷贝
     *
     * @param incomeRecords 收益记录信息集合
     * @return 收益记录信息列表
     */
    List<IncomeRecordDTO> copying(Collection<IncomeRecord> incomeRecords);

    /**
     * 收益记录信息拷贝
     *
     * @param param 收益记录更新参数
     * @return 收益记录信息
     */
    IncomeRecord copying(IncomeRecordModifyParam param);

    /**
     * 收益记录信息拷贝
     *
     * @param param    收益记录更新参数
     * @param incomeRecord 收益记录信息
     */
    void copying(IncomeRecordModifyParam param, @MappingTarget IncomeRecord incomeRecord);
}
