package com.chelvc.cloud.vehicle.server.copier;
import com.chelvc.cloud.vehicle.api.dto.MerchantAuthDTO;
import com.chelvc.cloud.vehicle.api.param.MerchantAuthModifyParam;
import com.chelvc.cloud.vehicle.server.entity.MerchantAuth;
import org.mapstruct.Builder;
import org.mapstruct.Mapper;
import org.mapstruct.MappingTarget;
import org.mapstruct.factory.Mappers;

import java.util.Collection;
import java.util.List;

/**
 * 商家认证拷贝接口
 *
 * @author liude
 * @date 2023/12/20
 */
@Mapper(builder = @Builder(disableBuilder = true))
public interface MerchantAuthCopier {
    /**
     * 对象拷贝接口实例
     */
    MerchantAuthCopier INSTANCE = Mappers.getMapper(MerchantAuthCopier.class);

    /**
     * 商家认证拷贝
     *
     * @param merchantAuth 商家认证
     * @return 商家认证
     */
    MerchantAuthDTO copying(MerchantAuth merchantAuth);

    /**
     * 商家认证信息拷贝
     *
     * @param merchantAuths 商家认证信息集合
     * @return 商家认证信息列表
     */
    List<MerchantAuthDTO> copying(Collection<MerchantAuth> merchantAuths);

    /**
     * 商家认证信息拷贝
     *
     * @param param 商家认证更新参数
     * @return 商家认证信息
     */
    MerchantAuth copying(MerchantAuthModifyParam param);

    /**
     * 商家认证信息拷贝
     *
     * @param param    商家认证更新参数
     * @param merchantAuth 商家认证信息
     */
    void copying(MerchantAuthModifyParam param, @MappingTarget MerchantAuth merchantAuth);
}
