package com.chelvc.cloud.vehicle.server.copier;
import com.chelvc.cloud.vehicle.api.dto.OmsOrderDTO;
import com.chelvc.cloud.vehicle.api.dto.OmsOrderItemDTO;
import com.chelvc.cloud.vehicle.api.param.OmsOrderModifyParam;
import com.chelvc.cloud.vehicle.server.entity.OmsOrder;
import com.chelvc.cloud.vehicle.server.entity.OmsOrderItem;
import org.mapstruct.Builder;
import org.mapstruct.Mapper;
import org.mapstruct.MappingTarget;
import org.mapstruct.factory.Mappers;

import java.util.Collection;
import java.util.List;

/**
 * 订单信息添加商品拷贝接口
 *
 * @author liude
 * @date 2023/11/20
 */
@Mapper(builder = @Builder(disableBuilder = true))
public interface OrderItemCopier {
    /**
     * 对象拷贝接口实例
     */
    OrderItemCopier INSTANCE = Mappers.getMapper(OrderItemCopier.class);

    /**
     * 订单信息添加商品拷贝
     *
     * @param omsOrderItem 订单添加商品信息
     * @return 订单添加商品信息
     */
    OmsOrderItemDTO copying(OmsOrderItem omsOrderItem);

    /**
     * 订单添加商品信息拷贝
     *
     * @param omsOrderItems 订单添加商品信息集合
     * @return 订单添加商品信息列表
     */
    List<OmsOrderItemDTO> copying(Collection<OmsOrderItem> omsOrderItems);

    /**
     * 订单添加商品信息拷贝
     *
     * @param param 订单添加商品更新参数
     * @return 订单添加商品信息
     */
    OmsOrderItem copying(OmsOrderModifyParam param);

    /**
     * 订单添加商品信息拷贝
     *
     * @param param    订单添加商品更新参数
     * @param omsOrder 订单添加商品信息
     */
    void copying(OmsOrderModifyParam param, @MappingTarget OmsOrder omsOrder);
}
