package com.chelvc.cloud.vehicle.server.copier;

import com.chelvc.cloud.vehicle.api.dto.OmsOrderDTO;
import com.chelvc.cloud.vehicle.api.dto.OmsOrderSettingDTO;
import com.chelvc.cloud.vehicle.api.param.OmsOrderModifyParam;
import com.chelvc.cloud.vehicle.api.param.OmsOrderSettingModifyParam;
import com.chelvc.cloud.vehicle.server.entity.OmsOrder;
import com.chelvc.cloud.vehicle.server.entity.OmsOrderSetting;
import org.mapstruct.Builder;
import org.mapstruct.Mapper;
import org.mapstruct.MappingTarget;
import org.mapstruct.factory.Mappers;

import java.util.Collection;
import java.util.List;

/**
 * 订单设置拷贝接口
 *
 * @author liude
 * @date 2023/11/20
 */
@Mapper(builder = @Builder(disableBuilder = true))
public interface OrderSettingCopier {
    /**
     * 对象拷贝接口实例
     */
    OrderSettingCopier INSTANCE = Mappers.getMapper(OrderSettingCopier.class);

    /**
     * 订单设置拷贝
     *
     * @param omsOrderSetting 订单设置
     * @return 订单设置
     */
    OmsOrderSettingDTO copying(OmsOrderSetting omsOrderSetting);

    /**
     * 订单设置拷贝
     *
     * @param omsOrderSettings 订单设置集合
     * @return 订单设置列表
     */
    List<OmsOrderSettingDTO> copying(Collection<OmsOrderSetting> omsOrderSettings);

    /**
     * 订单设置拷贝
     *
     * @param param 订单更新参数
     * @return 订单设置
     */
    OmsOrderSetting copying(OmsOrderSettingModifyParam param);

    /**
     * 订单设置拷贝
     *
     * @param param    订单更新参数
     * @param omsOrderSetting 订单设置
     */
    void copying(OmsOrderSettingModifyParam param, @MappingTarget OmsOrderSetting omsOrderSetting);
}
