package com.chelvc.cloud.vehicle.server.copier;

import com.chelvc.cloud.vehicle.api.dto.CouponDTO;
import com.chelvc.cloud.vehicle.api.dto.UserCouponDTO;
import com.chelvc.cloud.vehicle.api.param.UserCouponModifyParam;
import com.chelvc.cloud.vehicle.server.entity.Coupon;
import com.chelvc.cloud.vehicle.server.entity.UserCoupon;
import org.mapstruct.Builder;
import org.mapstruct.Mapper;
import org.mapstruct.Mapping;
import org.mapstruct.MappingTarget;
import org.mapstruct.factory.Mappers;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

/**
 * 优惠券领取对象拷贝接口
 *
 * @author qizai
 * @date 2023/9/8
 */
@Mapper(builder = @Builder(disableBuilder = true))
public interface UserCouponCopier {
    /**
     * 对象拷贝接口实例
     */
    UserCouponCopier INSTANCE = Mappers.getMapper(UserCouponCopier.class);

    /**
     * 优惠券领取信息拷贝
     *
     * @param userCoupon 优惠券领取信息
     * @return 优惠券领取信息
     */
    UserCouponDTO copying(UserCoupon userCoupon);

    /**
     * 优惠券领取信息拷贝
     *
     * @param param 优惠券领取信息更新参数
     * @return 优惠券领取信息
     */
    UserCoupon copying(UserCouponModifyParam param);

    /**
     * 优惠券领取信息拷贝
     *
     * @param param  优惠券领取信息更新参数
     * @param userCoupon 优惠券领取信息
     */
    void copying(UserCouponModifyParam param, @MappingTarget UserCoupon userCoupon);

    /**
     * 优惠券领取信息拷贝
     *
     * @param coupons 优惠券领取信息集合
     * @return 优惠券领取信息列表
     */
    List<UserCouponDTO> copying(Collection<UserCoupon> coupons);

    @Mapping(source = "userCoupon.id", target = "id")
    @Mapping(source = "userCoupon.type", target = "type")
    @Mapping(source = "coupon", target = "couponDTO")
    UserCouponDTO copying(UserCoupon userCoupon, Coupon coupon);
}
