package com.chelvc.cloud.vehicle.server.dao;

import java.util.Collection;
import java.util.List;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.chelvc.cloud.vehicle.api.dto.CouponDTO;
import com.chelvc.cloud.vehicle.server.entity.Coupon;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

/**
 * 优惠券数据操作接口
 *
 * @author Woody
 * @date 2023/7/17
 */
@Mapper
public interface CouponMapper extends BaseMapper<Coupon> {
    /**
     * 获取用户针对某个商品的可用优惠券
     *
     * @param userId         用户ID
     * @param goodsCouponIds 商品优惠券ID集合
     * @return 优惠券列表
     */
    List<CouponDTO> listUserGoodsActiveCoupons(@Param("userId") Long userId,
                                               @Param("goodsCouponIds") Collection<Long> goodsCouponIds);

    /**
     * 修改优惠券已领取数量
     *
     * @param id   优惠券ID
     * @param receiveNum 领取数量
     * @param publishNum 发行数量
     */
    int updateCouponReceivedNum(@Param("id") Long id, @Param("receiveNum") Integer receiveNum,
                                 @Param("publishNum") Integer publishNum);
}
