package com.chelvc.cloud.vehicle.server.dao;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Constants;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.chelvc.cloud.vehicle.api.dto.DynamicContentDTO;
import com.chelvc.cloud.vehicle.server.entity.DynamicContent;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Update;
import org.springframework.stereotype.Repository;

/**
 * @PACKAGE_NAME: net.yeeu.collision.dynamic.mapper
 * @NAME: DynamicContentMapper
 * @USER: igl
 * @DATE: 2023/8/8 15:24
 */
@Mapper
@Repository
public interface DynamicContentMapper extends BaseMapper<DynamicContent> {

  DynamicContentDTO queryById(@Param("id") Long id);

  @Update("update dynamic_content set comment_num = comment_num + 1 where id = #{dynamicId}")
  int addComment(Long dynamicId);

  @Update("update dynamic_content set comment_num = comment_num - #{num} where id = #{dynamicId}")
  int reduceComment(@Param("dynamicId")Long dynamicId, @Param("num")int num);

  @Update("update dynamic_content set good_num = good_num + 1 where id = #{contentId}")
  int addGoodNum(Long contentId);

  @Update("update dynamic_content set good_num = good_num - 1 where id = #{contentId}")
  int reduceGoodNum(Long contentId);

  @Update("update dynamic_content set bad_num = bad_num + 1 where id = #{contentId}")
  int addBadNum(Long contentId);

  @Update("update dynamic_content set bad_num = bad_num - 1 where id = #{contentId}")
  int reduceBadNum(Long contentId);

  @Update("update dynamic_content set platform_status = #{type} where id = #{id}")
  int shieldById(@Param("id")Long id, @Param("type")int type);

  @Update("update dynamic_content set platform_status = #{type} where deleted = 0 and user_id = #{userId}")
  int shieldByUserId(@Param("userId")Long userId, @Param("type")int type);

  IPage<DynamicContentDTO> selectAttention(@Param("page") Page<DynamicContentDTO> page, @Param(Constants.WRAPPER) QueryWrapper<DynamicContent> queryWrapper, @Param("userId") Long userId);

  IPage<DynamicContentDTO> queryHot(@Param("page") Page<DynamicContentDTO> page, @Param(Constants.WRAPPER) QueryWrapper<DynamicContent> queryWrapper);

  IPage<DynamicContentDTO> queryAll(@Param("page") Page<DynamicContentDTO> page, @Param(Constants.WRAPPER) QueryWrapper<DynamicContent> queryWrapper);

  IPage<DynamicContentDTO> queryBySelf(@Param("page") Page<DynamicContentDTO> page, @Param(Constants.WRAPPER) QueryWrapper<DynamicContent> queryWrapper);

  IPage<DynamicContentDTO> queryRandom(@Param("page") Page<DynamicContentDTO> page, @Param(Constants.WRAPPER) QueryWrapper<DynamicContent> queryWrapper);

}
