package com.chelvc.cloud.vehicle.server.dao;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.chelvc.cloud.vehicle.api.dto.OmsOrderDetailDTO;
import com.chelvc.cloud.vehicle.api.dto.OmsOrderExampleDTO;
import com.chelvc.cloud.vehicle.api.dto.OmsOrderItemDTO;
import com.chelvc.cloud.vehicle.server.entity.OmsOrder;
import com.chelvc.cloud.vehicle.server.entity.OmsOrderItem;
import com.chelvc.cloud.vehicle.server.entity.Reservation;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * 订单Mapper接口
 * 
 * @author liude
 * @date 2023-11-08
 */
@Mapper
public interface OmsOrderMapper extends BaseMapper<OmsOrder>
{
    /**
     * 查询订单
     * 
     * @param id 订单主键
     * @return 订单
     */
    public OmsOrder selectOmsOrderById(Long id);

    /**
     * 查询订单列表
     * 
     * @param omsOrder 订单
     * @return 订单集合
     */
    public List<OmsOrder> selectOmsOrderList(OmsOrder omsOrder);

    /**
     * 新增订单
     * 
     * @param omsOrder 订单
     * @return 结果
     */
    public int insertOmsOrder(OmsOrder omsOrder);

    /**
     * 修改订单
     * 
     * @param omsOrder 订单
     * @return 结果
     */
    public int updateOmsOrder(OmsOrder omsOrder);

    /**
     * 删除订单
     * 
     * @param id 订单主键
     * @return 结果
     */
    public int deleteOmsOrderById(Long id);

    /**
     * 批量删除订单
     * 
     * @param ids 需要删除的数据主键集合
     * @return 结果
     */
    public int deleteOmsOrderByIds(Long[] ids);


    long countByExample(OmsOrderExampleDTO example);

    int deleteByExample(OmsOrderExampleDTO example);

    int deleteByPrimaryKey(Long id);

    int insertSelective(OmsOrder record);

    List<OmsOrder> selectByExample(OmsOrderExampleDTO example);

    int updateByExampleSelective(@Param("record") OmsOrder record, @Param("example") OmsOrderExampleDTO example);

    int updateByExample(@Param("record") OmsOrder record, @Param("example") OmsOrderExampleDTO example);

    /**
     * 获取订单及下单商品详情
     */
    OmsOrderDetailDTO getDetail(@Param("orderId") Long orderId);

    /**
     * 修改 pms_sku_stock表的锁定库存及真实库存
     */
    int updateSkuStock(@Param("itemList") List<OmsOrderItemDTO> orderItemList);

    /**
     * 获取超时订单
     * @param minute 超时时间（分）
     */
    List<OmsOrderDetailDTO> getTimeOutOrders(@Param("minute") Integer minute);

    /**
     * 批量修改订单状态
     */
    int updateOrderStatus(@Param("ids") List<Long> ids,@Param("status") Integer status);

    /**
     * 解除取消订单的库存锁定
     */
    int releaseSkuStockLock(@Param("itemList") List<OmsOrderItem> orderItemList);
}
