package com.chelvc.cloud.vehicle.server.dao;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.chelvc.cloud.vehicle.api.dto.OmsOrderOperateHistoryDTO;
import com.chelvc.cloud.vehicle.server.entity.OmsOrderOperateHistory;
import org.apache.ibatis.annotations.Mapper;

import java.util.List;

/**
 * 订单操作历史记录Mapper接口
 * 
 * @author liude
 * @date 2023-11-08
 */
@Mapper
public interface OmsOrderOperateHistoryMapper extends BaseMapper<OmsOrderOperateHistory>
{
    /**
     * 查询订单操作历史记录
     * 
     * @param id 订单操作历史记录主键
     * @return 订单操作历史记录
     */
    public OmsOrderOperateHistory selectOmsOrderOperateHistoryById(Long id);

    /**
     * 查询订单操作历史记录列表
     * 
     * @param omsOrderOperateHistory 订单操作历史记录
     * @return 订单操作历史记录集合
     */
    public List<OmsOrderOperateHistory> selectOmsOrderOperateHistoryList(OmsOrderOperateHistory omsOrderOperateHistory);

    /**
     * 新增订单操作历史记录
     * 
     * @param omsOrderOperateHistory 订单操作历史记录
     * @return 结果
     */
    public int insertOmsOrderOperateHistory(OmsOrderOperateHistory omsOrderOperateHistory);

    /**
     * 修改订单操作历史记录
     * 
     * @param omsOrderOperateHistory 订单操作历史记录
     * @return 结果
     */
    public int updateOmsOrderOperateHistory(OmsOrderOperateHistory omsOrderOperateHistory);

    /**
     * 删除订单操作历史记录
     * 
     * @param id 订单操作历史记录主键
     * @return 结果
     */
    public int deleteOmsOrderOperateHistoryById(Long id);

    /**
     * 批量删除订单操作历史记录
     * 
     * @param ids 需要删除的数据主键集合
     * @return 结果
     */
    public int deleteOmsOrderOperateHistoryByIds(Long[] ids);

    /**
     * 根据用户查询订单通知信息
     *
     * @param userId 用户id
     * @return 订单操作历史记录
     */
    public List<OmsOrderOperateHistoryDTO> selectOmsOrderOperateHistoryListByUserId(Long userId);

    /**
     * 将新订单通知标识更改
     *
     * @param userId 用户id
     */
    public void changeUserOrderInformFlag(Long userId);
}
