package com.chelvc.cloud.vehicle.server.dao;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.chelvc.cloud.vehicle.api.param.UserCouponQueryParam;
import com.chelvc.cloud.vehicle.server.entity.UserCoupon;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * 优惠券领取数据操作接口
 *
 * @author Woody
 * @date 2023/7/17
 */
@Mapper
public interface UserCouponMapper extends BaseMapper<UserCoupon> {

    /**
     * 查询已过期的用户优惠券
     *
     * @param param 查询参数
     * @return 用户优惠券信息列表
     */
    List<UserCoupon> listExpiredUserCoupons(@Param("userId") Long userId, @Param("param") UserCouponQueryParam param);

    /**
     * 查询用户优惠券
     *
     * @param param 查询参数
     * @return 用户优惠券信息列表
     */
    List<UserCoupon> listUserCoupons(@Param("userId") Long userId, @Param("param") UserCouponQueryParam param);

    /**
     * 获取当前用户对于当前商品可使用的优惠券列表
     *
     * @param param 查询参数
     * @return 用户可用优惠券信息列表
     */
    List<UserCoupon> listUserCanUseCoupons(@Param("userId") Long userId, @Param("param") UserCouponQueryParam param);
}
