package com.chelvc.cloud.vehicle.server.service;

import com.chelvc.cloud.vehicle.api.dto.OmsOrderOperateHistoryDTO;
import com.chelvc.cloud.vehicle.server.entity.OmsOrderOperateHistory;
import com.chelvc.framework.common.model.Pagination;
import com.chelvc.framework.common.model.Paging;

import java.util.List;

/**
 * 订单操作历史记录Service接口
 * 
 * @author liude
 * @date 2023-11-08
 */
public interface IOmsOrderOperateHistoryService 
{
    /**
     * 查询订单操作历史记录
     * 
     * @param id 订单操作历史记录主键
     * @return 订单操作历史记录
     */
    public OmsOrderOperateHistory selectOmsOrderOperateHistoryById(Long id);

    /**
     * 查询订单操作历史记录列表
     * 
     * @param omsOrderOperateHistory 订单操作历史记录
     * @return 订单操作历史记录集合
     */
    public List<OmsOrderOperateHistory> selectOmsOrderOperateHistoryList(OmsOrderOperateHistory omsOrderOperateHistory);

    /**
     * 新增订单操作历史记录
     * 
     * @param omsOrderOperateHistory 订单操作历史记录
     * @return 结果
     */
    public int insertOmsOrderOperateHistory(OmsOrderOperateHistory omsOrderOperateHistory);

    /**
     * 修改订单操作历史记录
     * 
     * @param omsOrderOperateHistory 订单操作历史记录
     * @return 结果
     */
    public int updateOmsOrderOperateHistory(OmsOrderOperateHistory omsOrderOperateHistory);

    /**
     * 批量删除订单操作历史记录
     * 
     * @param ids 需要删除的订单操作历史记录主键集合
     * @return 结果
     */
    public int deleteOmsOrderOperateHistoryByIds(Long[] ids);

    /**
     * 删除订单操作历史记录信息
     * 
     * @param id 订单操作历史记录主键
     * @return 结果
     */
    public int deleteOmsOrderOperateHistoryById(Long id);

    /**
     * 根据用户查询订单通知信息
     *
     * @param userId 用户id
     * @param paging 分页信息
     * @return 订单操作历史记录
     */
    Pagination<OmsOrderOperateHistoryDTO> selectOmsOrderOperateHistoryListByUserId(Long userId, Paging paging);

    /**
     * 将新订单通知标识更改
     *
     * @param userId 用户id
     */
    void changeUserOrderInformFlag(Long userId);

    /**
     * 查询订单操作历史数量
     * @param userId
     * @return
     */
    Integer queryNewInformNum(Long userId);
}
