package com.chelvc.cloud.vehicle.server.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.chelvc.cloud.uc.api.dto.UserDTO;
import com.chelvc.cloud.uc.api.service.UserService;
import com.chelvc.cloud.vehicle.api.dto.CommentDTO;
import com.chelvc.cloud.vehicle.api.param.CommentQueryParam;
import com.chelvc.cloud.vehicle.server.dao.CommentMapper;
import com.chelvc.cloud.vehicle.server.entity.Comment;
import com.chelvc.cloud.vehicle.server.service.CommentService;
import com.chelvc.framework.common.model.Pagination;
import com.chelvc.framework.common.model.Paging;
import com.chelvc.framework.common.util.BeanCopyUtil;
import com.chelvc.framework.database.util.PagingUtils;
import lombok.NonNull;
import org.apache.dubbo.config.annotation.DubboReference;
import org.apache.dubbo.config.annotation.DubboService;
import org.springframework.util.CollectionUtils;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 商家评论业务操作实现
 *
 * @author xp
 * @data 2023/3/31
 */
@DubboService(interfaceClass = com.chelvc.cloud.vehicle.api.service.CommentService.class)
public class CommentServiceImpl extends ServiceImpl<CommentMapper, Comment> implements CommentService,
        com.chelvc.cloud.vehicle.api.service.CommentService {
    @DubboReference
    private UserService userService;

    @Override
    public List<CommentDTO> listGoodsComments(@NonNull Long goodsId, @NonNull CommentQueryParam param) {
        // 获取商品评价列表
        List<CommentDTO> comments = this.baseMapper.listGoodsComments(goodsId, param);

        // 加载评价用户信息
        Set<Long> userIds = CollectionUtils.isEmpty(comments) ? Collections.emptySet() :
                comments.stream().map(CommentDTO::getUserId).filter(Objects::nonNull).collect(Collectors.toSet());
        if (!CollectionUtils.isEmpty(userIds)) {
            Map<Long, UserDTO> users = this.userService.listUsers(userIds).stream()
                    .collect(Collectors.toMap(UserDTO::getId, user -> user));
            comments.forEach(comment -> comment.setUser(users.get(comment.getUserId())));
        }
        return comments;
    }

    @Override
    public Integer queryNewInformNum(Long userId) {
        LambdaQueryWrapper<Comment> lwq = Wrappers.lambdaQuery();
        lwq.eq(Comment::getUserId, userId);
        lwq.eq(Comment::getFlag, 0);
        return baseMapper.selectCount(lwq);
    }

    @Override
    public void changeCommentInformFlag(Long userId) {
        LambdaUpdateWrapper<Comment> lwq = Wrappers.lambdaUpdate();
        lwq.eq(Comment::getUserId, userId);
        lwq.eq(Comment::getFlag, 0);
        lwq.set(Comment::getFlag, 1);
        baseMapper.update(null, lwq);
    }

    @Override
    public Pagination<CommentDTO> selectCommentListByUserId(Long userId, Paging paging) {
        Page<Comment> page = this.lambdaQuery()
                .eq(Comment::getUserId, userId)
                .orderByAsc(Comment::getCreateTime).page(PagingUtils.convert(paging));
        List<Comment> comments = page.getRecords();
        List<CommentDTO> copy = BeanCopyUtil.copy(comments, CommentDTO.class);
        return PagingUtils.convert(page, copy);
    }
}
