package com.chelvc.cloud.vehicle.server.service.impl;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.chelvc.cloud.vehicle.api.dto.CommissionConfigDTO;
import com.chelvc.cloud.vehicle.api.param.CommissionConfigModifyParam;
import com.chelvc.cloud.vehicle.api.param.CommissionConfigPagingParam;
import com.chelvc.cloud.vehicle.server.copier.CommissionConfigCopier;
import com.chelvc.cloud.vehicle.server.dao.CommissionConfigMapper;
import com.chelvc.cloud.vehicle.server.entity.CommissionConfig;
import com.chelvc.cloud.vehicle.server.service.CommissionConfigService;
import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.base.util.ResourceUtils;
import com.chelvc.framework.common.model.Pagination;
import com.chelvc.framework.database.util.PagingUtils;
import lombok.NonNull;
import org.apache.dubbo.config.annotation.DubboService;

import java.util.Date;

/**
 * 抽成业务操作实现
 *
 * @author liude
 * @date 2023/12/16
 */
@DubboService(interfaceClass = com.chelvc.cloud.vehicle.api.service.CommissionConfigService.class)
public class CommissionConfigServiceImpl extends ServiceImpl<CommissionConfigMapper, CommissionConfig> implements CommissionConfigService,
        com.chelvc.cloud.vehicle.api.service.CommissionConfigService {
    @Override
    public Long addCommissionConfig(@NonNull CommissionConfigModifyParam param) {
        CommissionConfig commissionConfig = CommissionConfigCopier.INSTANCE.copying(param);
        commissionConfig.setCreateTime(new Date());
        commissionConfig.setUpdateTime(new Date());
        Long userId = SessionContextHolder.getId();
        commissionConfig.setCreator(userId);
        commissionConfig.setUpdater(userId);
        this.save(commissionConfig);
        return commissionConfig.getId();
    }

    @Override
    public void updateCommissionConfig(@NonNull Long id, @NonNull CommissionConfigModifyParam param) {
        CommissionConfig commissionConfig = ResourceUtils.required(this.getById(id), "该轮播图不存在");
        CommissionConfigCopier.INSTANCE.copying(param, commissionConfig);
        Long userId = SessionContextHolder.getId();
        commissionConfig.setUpdateTime(new Date());
        commissionConfig.setUpdater(userId);
        this.updateById(commissionConfig);
    }

    @Override
    public Pagination<CommissionConfigDTO> getCommissionConfigPaging(@NonNull CommissionConfigPagingParam param) {
        Page<CommissionConfig> page = this.lambdaQuery()
                .orderByAsc(CommissionConfig::getCommissionRate).page(PagingUtils.convert(param.getPaging()));
        return PagingUtils.convert(page, CommissionConfigCopier.INSTANCE::copying);
    }

    @Override
    public void deleteCommissionConfig(Long id){
        ResourceUtils.required(this.getById(id), "该轮播图存在");
        this.baseMapper.deleteById(id);
    }


 }
