package com.chelvc.cloud.vehicle.server.service.impl;

import com.baomidou.lock.annotation.Lock4j;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.chelvc.cloud.uc.api.dto.UserDTO;
import com.chelvc.cloud.uc.api.service.UserService;
import com.chelvc.cloud.vehicle.api.constant.DynamicStatusType;
import com.chelvc.cloud.vehicle.api.constant.ExamineType;
import com.chelvc.cloud.vehicle.api.dto.DynamicCommentDTO;
import com.chelvc.cloud.vehicle.api.param.AddDynamicCommentParam;
import com.chelvc.cloud.vehicle.api.param.QueryAdminDynamicParam;
import com.chelvc.cloud.vehicle.api.param.QueryDynamicCommentParam;
import com.chelvc.cloud.vehicle.server.dao.DynamicCommentMapper;
import com.chelvc.cloud.vehicle.server.entity.DynamicComment;
import com.chelvc.cloud.vehicle.server.service.DynamicCommentService;
import com.chelvc.cloud.vehicle.server.service.DynamicContentService;
import com.chelvc.framework.base.exception.ResourceUnavailableException;
import com.chelvc.framework.common.model.Pagination;
import com.chelvc.framework.common.util.BeanCopyUtil;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.DubboReference;
import org.apache.dubbo.config.annotation.DubboService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @PACKAGE_NAME: net.yeeu.collision.dynamic.service.impl
 * @NAME: DynamicCommentServiceImpl
 * @USER: igl
 * @DATE: 2023/8/8 18:53
 */
@Slf4j
@Service
@RequiredArgsConstructor(onConstructor = @__({@Autowired, @Lazy}))
@DubboService(interfaceClass = com.chelvc.cloud.vehicle.api.service.DynamicCommentService.class)
public class DynamicCommentServiceImpl implements DynamicCommentService, com.chelvc.cloud.vehicle.api.service.DynamicCommentService {
  private final DynamicCommentMapper baseMapper;
  private final DynamicContentService dynamicContentService;
  @DubboReference
  private final UserService userService;

  //private final ContentAuditHelper contentAuditHelper;
  /**
   * 查询动态评论列表
   */
  @Override
  public Pagination<DynamicCommentDTO> queryPageList(QueryDynamicCommentParam param, Integer pageNum,
                                                     Integer pageSize) {
    LambdaQueryWrapper<DynamicComment> lqw = Wrappers.lambdaQuery();
    lqw.eq(DynamicComment::getDynamicId, param.getDynamicId());
    lqw.eq(DynamicComment::getParentId, param.getParentId());
    lqw.eq(DynamicComment::getPlatformStatus, 0);
    Long userId = param.getUserId();
    lqw.and(wq -> wq
        .eq(DynamicComment::getExamineStatus, 1)
        .or(wq1 -> wq1
            .eq(DynamicComment::getExamineStatus, 0)
            .eq(DynamicComment::getUserId, userId)));
    lqw.orderByDesc(DynamicComment::getCreateTime);
    Page<DynamicComment> page = baseMapper.selectPage(
        new Page<>(pageNum, pageSize), lqw);
    List<DynamicComment> result = page.getRecords();
    if (result != null && result.size() > 0) {
      List<DynamicCommentDTO> records = BeanCopyUtil.copy(result,
          DynamicCommentDTO.class);
      List<Long> userIds = records.stream().map(DynamicCommentDTO::getUserId).collect(
          Collectors.toList());
      Map<Long, UserDTO> map = userService.queryUserByIds(userIds);
      records.forEach(e -> {
        e.setUserDTO(map == null ? new UserDTO() : map.get(e.getUserId()));
      });
      return Pagination.<DynamicCommentDTO>builder().total(page.getTotal()).pages(page.getPages())
              .records(records).build();
    }
    return new Pagination<>();
  }

  @Override
  public Pagination<DynamicCommentDTO> queryAdminPageList(QueryAdminDynamicParam param, Long pageNum,
                                                       Long pageSize) {
    Page<DynamicCommentDTO> page = new Page<>(pageNum, pageSize);
    IPage<DynamicCommentDTO> result = null;
    if (param == null) {
      return new Pagination<>();
    }
    result = baseMapper
        .queryAll(page, queryManagerWrapper(param));
    IPage<DynamicCommentDTO> resultDto = getResult(result);
    return Pagination.<DynamicCommentDTO>builder().total(result.getTotal()).pages(result.getPages())
            .records(resultDto.getRecords()).build();
  }

  public IPage<DynamicCommentDTO> getResult(IPage<DynamicCommentDTO> result) {
    if (result == null) {
      return null;
    }
    List<DynamicCommentDTO> records = result.getRecords();
    if (records != null && !records.isEmpty()) {
      List<Long> userIds = records.stream().map(DynamicCommentDTO::getUserId).collect(
          Collectors.toList());
      Map<Long, UserDTO> longUserBaseInfoMap = userService.queryUserByIds(
          userIds);
      log.info("远程查询用户信息成功:{}", longUserBaseInfoMap);
      List<String> resourceKey = new ArrayList<>();
      records.forEach(e -> {
        e.setUserDTO(longUserBaseInfoMap == null ? new UserDTO() :
            longUserBaseInfoMap.get(e.getUserId()));
      });
    }
    return result;
  }

  private QueryWrapper<DynamicComment> queryManagerWrapper(
      QueryAdminDynamicParam param) {
    QueryWrapper<DynamicComment> lqw = Wrappers.query();
    lqw.eq("t1.deleted", 0);
    lqw.like(StringUtils.isNotBlank(param.getKeyWord()), "t1.content", param.getKeyWord());
    lqw.ge(StringUtils.isNotBlank(param.getStartTime()), "t1.create_time", param.getStartTime());
    lqw.le(StringUtils.isNotBlank(param.getEndTime()), "t1.create_time", param.getEndTime());
    lqw.eq(param.getExamineStatus() != null, "t1.examine_status", param.getExamineStatus());
    lqw.eq(param.getExamineResult() != null, "t1.examine_result", param.getExamineResult());
    lqw.eq(param.getPlatformStatus() != null, "t1.platform_status", param.getPlatformStatus());
    lqw.orderByDesc("t1.create_time","t1.id");
    return lqw;
  }

  /**
   * 新增动态评论
   */
  @Override
  public void insertByBo(AddDynamicCommentParam param, Long userId) {
    DynamicComment add = BeanCopyUtil.copy(param, DynamicComment.class);
    add.setUserId(userId);
    boolean b = baseMapper.insert(add) > 0;
    if (!b) {
      throw new ResourceUnavailableException("新增评论失败");
    }
    //异步处理审核
    //contentAuditHelper.asyncCommentHandle(add);
  }

  private void addParentComment(Long parentId) {
    if(parentId == 0){
      return;
    }
    DynamicComment DynamicComment = baseMapper.selectById(parentId);
    if(DynamicComment == null){
      return;
    }
    int i = baseMapper.addComment(DynamicComment.getId());
    if(i != 1){
      throw new ResourceUnavailableException("新增评论失败");
    }
    addParentComment(DynamicComment.getParentId());
  }

  @Override
  @Lock4j(keys = {"#contentId"})
  public void addNum(Integer dynamicStatus, Long contentId) {
    if (Objects.equals(DynamicStatusType.GOOD.getValue(), dynamicStatus)) {
      int i = baseMapper.addGoodNum(contentId);
      if(i != 1){
        throw new ResourceUnavailableException("点赞失败");
      }
    } else if (Objects.equals(DynamicStatusType.BAD.getValue(), dynamicStatus)) {
      int i = baseMapper.addBadNum(contentId);
      if(i != 1){
        throw new ResourceUnavailableException("D踩一踩失败");
      }
    }
  }

  @Override
  @Lock4j(keys = {"#contentId"})
  public void reduceNum(Integer dynamicStatus, Long contentId) {
    if (Objects.equals(DynamicStatusType.GOOD.getValue(), dynamicStatus)) {
      int i = baseMapper.reduceGoodNum(contentId);
      if(i != 1){
        throw new ResourceUnavailableException("取消点赞失败");
      }
    } else if (Objects.equals(DynamicStatusType.BAD.getValue(), dynamicStatus)) {
      int i = baseMapper.reduceBadNum(contentId);
      if(i != 1){
        throw new ResourceUnavailableException("取消踩一踩失败");
      }
    }
  }

  @Override
  @Transactional(rollbackFor = RuntimeException.class)
  public void examine(Long id, ExamineType examine) {
    DynamicComment DynamicComment = baseMapper.selectById(id);
    if(DynamicComment == null){
      throw new ResourceUnavailableException("评论不存在");
    }
    LambdaUpdateWrapper<DynamicComment> lwq = Wrappers.lambdaUpdate();
    lwq.eq(com.chelvc.cloud.vehicle.server.entity.DynamicComment::getId, id);
    lwq.set(com.chelvc.cloud.vehicle.server.entity.DynamicComment::getExamineStatus, 1);
    if(ExamineType.REFUSE.equals(examine)){
      lwq.set(com.chelvc.cloud.vehicle.server.entity.DynamicComment::getPlatformStatus, 1);
    }
    int update = baseMapper.update(null, lwq);
    if(update != 1){
      throw new ResourceUnavailableException("操作失败");
    }
    if(ExamineType.PASS.equals(examine)){
      addParentComment(DynamicComment.getParentId());
      dynamicContentService.addComment(DynamicComment.getDynamicId());
    }
  }

  @Override
  public void autoAudit(Long id, int platformStatus, int examineResult) {
    LambdaUpdateWrapper<DynamicComment> lwq = Wrappers.lambdaUpdate();
    lwq.set(DynamicComment::getPlatformStatus, platformStatus);
    lwq.set(DynamicComment::getExamineStatus, 1);
    lwq.set(DynamicComment::getExamineResult, examineResult);
    lwq.set(DynamicComment::getUpdateTime, LocalDateTime.now());
    lwq.eq(DynamicComment::getId, id);
    int update = baseMapper.update(null, lwq);
    if(update != 1){
      throw new ResourceUnavailableException("自动审核失败");
    }
    if(ExamineType.PASS.getValue() == examineResult){
      DynamicComment DynamicComment = baseMapper.selectById(id);
      addParentComment(DynamicComment.getParentId());
      dynamicContentService.addComment(DynamicComment.getDynamicId());
    }
  }

  @Override
  public DynamicCommentDTO queryById(Long id) {
    DynamicComment dynamicComment = baseMapper.selectById(id);
    if(dynamicComment == null){
      return null;
    }
    return BeanCopyUtil.copy(dynamicComment, DynamicCommentDTO.class);
  }
}
