package com.chelvc.cloud.vehicle.server.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.chelvc.cloud.vehicle.api.constant.ActionType;
import com.chelvc.cloud.vehicle.api.constant.RecordType;
import com.chelvc.cloud.vehicle.api.dto.DynamicCommentDTO;
import com.chelvc.cloud.vehicle.api.dto.DynamicEvaluateRecordDTO;
import com.chelvc.cloud.vehicle.api.dto.EvaluateDTO;
import com.chelvc.cloud.vehicle.api.param.AddDynamicEvaluateRecordParam;
import com.chelvc.cloud.vehicle.server.dao.DynamicEvaluateRecordMapper;
import com.chelvc.cloud.vehicle.server.entity.DynamicEvaluateRecord;
import com.chelvc.cloud.vehicle.server.service.DynamicCommentService;
import com.chelvc.cloud.vehicle.server.service.DynamicContentService;
import com.chelvc.cloud.vehicle.server.service.DynamicEvaluateRecordService;
import com.chelvc.framework.base.exception.ResourceUnavailableException;
import com.chelvc.framework.common.model.Pagination;
import com.chelvc.framework.common.model.Paging;
import com.chelvc.framework.common.util.BeanCopyUtil;
import com.chelvc.framework.database.util.PagingUtils;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.List;
import java.util.Objects;

/**
 * @PACKAGE_NAME: net.yeeu.collision.dynamic.service.impl
 * @NAME: DynamicEvaluateRecordServiceImpl
 * @USER: igl
 * @DATE: 2023/8/8 18:40
 */
@Slf4j
@Service
@RequiredArgsConstructor(onConstructor = @__({@Autowired, @Lazy}))
public class DynamicEvaluateRecordServiceImpl extends ServiceImpl<DynamicEvaluateRecordMapper, DynamicEvaluateRecord> implements DynamicEvaluateRecordService {

  private final DynamicEvaluateRecordMapper baseMapper;

  private final DynamicContentService iDynamicContentService;

  private final DynamicCommentService iDynamicCommentService;

  private LambdaQueryWrapper<DynamicEvaluateRecord> buildAddWrapper(Long contentId, Long userId) {
    LambdaQueryWrapper<DynamicEvaluateRecord> lqw = Wrappers.lambdaQuery();
    lqw.eq(DynamicEvaluateRecord::getContentId, contentId);
    lqw.eq(DynamicEvaluateRecord::getUserId, userId);
    return lqw;
  }

  /**
   * 新增内容评价(点赞/踩)列
   */
  @Override
  @Transactional(rollbackFor = Exception.class)
  public Boolean updateRecord(AddDynamicEvaluateRecordParam param, Long userId) {
    Integer actionType = param.getActionType().getValue();
    Integer recordType = param.getRecordType().getValue();
    Integer dynamicStatusType = param.getDynamicStatusType().getValue();
    LambdaQueryWrapper<DynamicEvaluateRecord> lqw =
        buildAddWrapper(param.getContentId(), userId);
    if(Objects.equals(ActionType.ADD.getValue(), actionType)){
      //判断该用户是否有点赞/踩记录，点击新增记录，取消删除记录
      executeWrapper(lqw, param, true, userId);
      if (Objects.equals(RecordType.COMMENT.getValue(), recordType)) {
        iDynamicCommentService.addNum(dynamicStatusType, param.getContentId());
      } else if(Objects.equals(RecordType.DYNAMIC.getValue(), recordType)){
        iDynamicContentService.addNum(dynamicStatusType, param.getContentId());
      }
    } else if (Objects.equals(ActionType.CANCEL.getValue(), actionType)) {
      lqw.eq(DynamicEvaluateRecord::getOperaterType, dynamicStatusType);
      //判断该用户是否有点赞/踩记录，点击新增记录，取消删除记录
      executeWrapper(lqw, param, false, userId);
      if (Objects.equals(RecordType.COMMENT.getValue(), recordType)) {
        iDynamicCommentService.reduceNum(dynamicStatusType, param.getContentId());
      } else if(Objects.equals(RecordType.DYNAMIC.getValue(), recordType)){
        iDynamicContentService.reduceNum(dynamicStatusType, param.getContentId());
      }
    }
    return true;
  }

  @Override
  public DynamicEvaluateRecordDTO queryOperaterType(Long id, Long userId, Integer recordType) {
    LambdaQueryWrapper<DynamicEvaluateRecord> lwq = Wrappers.lambdaQuery();
    lwq.eq(DynamicEvaluateRecord::getContentId, id);
    lwq.eq(DynamicEvaluateRecord::getUserId, userId);
    lwq.eq(DynamicEvaluateRecord::getContentType, recordType);
    lwq.orderByDesc(DynamicEvaluateRecord::getCreateTime);
    List<DynamicEvaluateRecord> dynamicEvaluateRecordEntities = baseMapper.selectList(lwq);
    if (dynamicEvaluateRecordEntities != null && dynamicEvaluateRecordEntities.size() > 0){
      DynamicEvaluateRecord entity = dynamicEvaluateRecordEntities.get(0);
      return BeanCopyUtil.copy(entity, DynamicEvaluateRecordDTO.class);
    }
    return null;
  }

  @Override
  public Integer queryNewInformNum(Long userId) {
    LambdaQueryWrapper<DynamicEvaluateRecord> lwq = Wrappers.lambdaQuery();
    lwq.eq(DynamicEvaluateRecord::getTargetUserId, userId);
    lwq.eq(DynamicEvaluateRecord::getFlag, 0);
    return baseMapper.selectCount(lwq);
  }

  @Override
  public void changeEvaluateInformFlag(Long userId) {
    LambdaUpdateWrapper<DynamicEvaluateRecord> lwq = Wrappers.lambdaUpdate();
    lwq.eq(DynamicEvaluateRecord::getTargetUserId, userId);
    lwq.eq(DynamicEvaluateRecord::getFlag, 0);
    lwq.set(DynamicEvaluateRecord::getFlag, 1);
    baseMapper.update(null, lwq);
  }

  @Override
  public Pagination<EvaluateDTO> selectEvaluateListByUserId(Long userId, Paging paging) {
    Page<DynamicEvaluateRecord> page = this.lambdaQuery()
            .eq(DynamicEvaluateRecord::getTargetUserId, userId)
            .orderByAsc(DynamicEvaluateRecord::getCreateTime).page(PagingUtils.convert(paging));
    List<DynamicEvaluateRecord> dynamicEvaluateRecords = page.getRecords();
    List<EvaluateDTO> copy = BeanCopyUtil.copy(dynamicEvaluateRecords, EvaluateDTO.class);
    if(copy != null && copy.size() > 0){
      for (EvaluateDTO evaluateDTO : copy) {
        Integer contentType = evaluateDTO.getContentType();
        if(contentType == 1){
            evaluateDTO.setDynamicContentDTO(iDynamicContentService.queryById(evaluateDTO.getContentId(), evaluateDTO.getUserId()));
        } else {
          DynamicCommentDTO dynamicCommentDTO = iDynamicCommentService.queryById(evaluateDTO.getContentId());
          if(dynamicCommentDTO != null){
            evaluateDTO.setDynamicContentDTO(iDynamicContentService.queryById(dynamicCommentDTO.getDynamicId(), evaluateDTO.getUserId()));
          }
          evaluateDTO.setDynamicCommentDTO(dynamicCommentDTO);
        }
      }
    }
    return PagingUtils.convert(page, copy);
  }

  /**
   * 判断该用户是否有点赞/踩记录，点击新增记录，取消删除记录
   * @param lqw
   * @param param
   * @param b
   */
  public void executeWrapper(LambdaQueryWrapper<DynamicEvaluateRecord> lqw,
      AddDynamicEvaluateRecordParam param, boolean b, Long userId){
    DynamicEvaluateRecord DynamicEvaluateRecord = baseMapper.selectOne(lqw);
    if(b){
      if(DynamicEvaluateRecord != null){
        throw new ResourceUnavailableException("处理失败");
      }
      DynamicEvaluateRecord contentEvaluateRecord = BeanCopyUtil.copy(param,
          DynamicEvaluateRecord.class);
      contentEvaluateRecord.setContentType(param.getRecordType().getValue());
      contentEvaluateRecord.setOperaterType(param.getDynamicStatusType().getValue());
      contentEvaluateRecord.setUserId(userId);
      contentEvaluateRecord.setFlag(0);
      contentEvaluateRecord.setCreateTime(new Date());
      baseMapper.insert(contentEvaluateRecord);
    } else {
      if(DynamicEvaluateRecord == null){
        throw new ResourceUnavailableException("处理失败");
      }
      baseMapper.deleteByRecordId(DynamicEvaluateRecord.getId());
    }
    log.info("用户:[{}]{}{}{}",userId, param.getRecordType().getDesc(),
        param.getActionType().getDesc(), param.getDynamicStatusType().getDesc());
  }
}
