package com.chelvc.cloud.vehicle.server.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.chelvc.cloud.vehicle.api.dto.DynamicResourceDTO;
import com.chelvc.cloud.vehicle.server.dao.DynamicResourceMapper;
import com.chelvc.cloud.vehicle.server.entity.DynamicResource;
import com.chelvc.cloud.vehicle.server.service.DynamicResourceService;
import com.chelvc.framework.base.exception.ResourceUnavailableException;
import com.chelvc.framework.common.util.BeanCopyUtil;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Collections;
import java.util.List;

/**
 * 动态文件Service业务层处理
 *
 * @author igl
 * @date 2023-02-25
 */
@Service
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class DynamicResourceServiceImpl implements DynamicResourceService {

    private final DynamicResourceMapper baseMapper;

    @Override
    public void batchInsert(List<DynamicResource> list) {
        int b = baseMapper.insertBatchSomeColumn(list);
        if(b != list.size()){
            throw new ResourceUnavailableException("保存动态资源失败");
        }
    }

    @Override
    public List<DynamicResourceDTO> queryByDynamicId(Long dynamicId) {
        LambdaQueryWrapper<DynamicResource> lqw = Wrappers.lambdaQuery();
        lqw.eq(DynamicResource::getDynamicId, dynamicId);
        lqw.orderByAsc(DynamicResource::getSort);
        List<DynamicResource> list = baseMapper.selectList(lqw);
        if(list == null || list.size() == 0){
            return Collections.emptyList();
        }
        return BeanCopyUtil.copy(list, DynamicResourceDTO.class);
    }
}
