package com.chelvc.cloud.vehicle.server.service.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.chelvc.cloud.vehicle.api.constant.FavoriteType;
import com.chelvc.cloud.vehicle.api.dto.FavoriteDTO;
import com.chelvc.cloud.vehicle.api.dto.MerchantDTO;
import com.chelvc.cloud.vehicle.api.param.FavoritePagingParam;
import com.chelvc.cloud.vehicle.api.param.FavoriteQueryParam;
import com.chelvc.cloud.vehicle.server.copier.FavoriteCopier;
import com.chelvc.cloud.vehicle.server.dao.FavoriteMapper;
import com.chelvc.cloud.vehicle.server.entity.Favorite;
import com.chelvc.cloud.vehicle.server.service.FavoriteService;
import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.base.util.ResourceUtils;
import com.chelvc.framework.common.model.Pagination;
import lombok.NonNull;
import org.apache.dubbo.config.annotation.DubboService;
import org.springframework.util.CollectionUtils;

import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * 用户收藏业务操作实现
 *
 * @author liude
 * @date 2023/7/20
 */
@DubboService(interfaceClass = com.chelvc.cloud.vehicle.api.service.FavoriteService.class)
public class FavoriteServiceImpl extends ServiceImpl<FavoriteMapper, Favorite> implements FavoriteService,
        com.chelvc.cloud.vehicle.api.service.FavoriteService {
    @Override
    public Long addMerchantFavorite(@NonNull Long merchantId) {
        Long userId = SessionContextHolder.getId();
        Integer count = this.lambdaQuery()
                .eq(Favorite::getUserId, userId)
                .eq(Favorite::getType, FavoriteType.MERCHANT)
                .eq(Favorite::getContentId, merchantId).count();
        ResourceUtils.available(count <= 0, "收藏信息已存在");
        Favorite favorite = Favorite.builder().userId(userId).type(FavoriteType.MERCHANT)
                .contentId(merchantId).createTime(new Date()).build();
        this.baseMapper.saveFavorite(favorite);
        return favorite.getId();
    }

    @Override
    public void deleteMineFavorite(@NonNull Long id) {
        Favorite favorite = ResourceUtils.required(this.getById(id), "收藏信息不存在");
        ResourceUtils.available(Objects.equals(favorite.getUserId(), SessionContextHolder.getId()), "非法操作");
        this.removeById(id);
    }

    @Override
    public Pagination<MerchantDTO> getFavoriteMerchantPaging(FavoritePagingParam param) {
        return null;
    }

    @Override
    public List<FavoriteDTO> listFavoriteMerchants(@NonNull FavoriteQueryParam param) {
        Long userId = SessionContextHolder.getId();
        List<Favorite> favorites = this.baseMapper.listFavoriteMerchants(userId, param);
        if (CollectionUtils.isEmpty(favorites)) {
            return Collections.emptyList();
        }
        return favorites
                .stream()
                .map(o -> {
                    FavoriteDTO favoriteDTO = FavoriteCopier.INSTANCE.copying(o);
                    favoriteDTO.setMerchant(o.getMerchant());
                    return favoriteDTO;
                }).collect(Collectors.toList());

    }
}
