package com.chelvc.cloud.vehicle.server.service.impl;

import java.util.List;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.chelvc.cloud.vehicle.api.constant.CategoryType;
import com.chelvc.cloud.vehicle.api.constant.GoodsStatus;
import com.chelvc.cloud.vehicle.api.constant.MerchantStatus;
import com.chelvc.cloud.vehicle.api.dto.CouponDTO;
import com.chelvc.cloud.vehicle.api.dto.FavoriteDTO;
import com.chelvc.cloud.vehicle.api.dto.GoodsDTO;
import com.chelvc.cloud.vehicle.api.dto.GoodsDetailDTO;
import com.chelvc.cloud.vehicle.api.param.*;
import com.chelvc.cloud.vehicle.server.copier.CouponCopier;
import com.chelvc.cloud.vehicle.server.copier.GoodsCopier;
import com.chelvc.cloud.vehicle.server.dao.GoodsMapper;
import com.chelvc.cloud.vehicle.server.entity.Coupon;
import com.chelvc.cloud.vehicle.server.entity.Favorite;
import com.chelvc.cloud.vehicle.server.entity.Goods;
import com.chelvc.cloud.vehicle.server.entity.Merchant;
import com.chelvc.cloud.vehicle.server.service.CommentService;
import com.chelvc.cloud.vehicle.server.service.CouponService;
import com.chelvc.cloud.vehicle.server.service.GoodsService;
import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.base.util.ResourceUtils;
import com.chelvc.framework.common.model.Pagination;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.database.context.DatabaseContextHolder;
import com.chelvc.framework.database.util.PagingUtils;
import lombok.NonNull;
import lombok.RequiredArgsConstructor;
import org.apache.dubbo.config.annotation.DubboService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;

/**
 * 商品业务操作实现
 *
 * @author Woody
 * @date 2023/7/17
 */
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
@DubboService(interfaceClass = com.chelvc.cloud.vehicle.api.service.GoodsService.class)
public class GoodsServiceImpl extends ServiceImpl<GoodsMapper, Goods> implements GoodsService,
        com.chelvc.cloud.vehicle.api.service.GoodsService {
    private final CouponService couponService;
    private final CommentService commentService;

    @Override
    public List<CategoryType> listMerchantGoodsCategoryTypes(@NonNull Long merchantId) {
        List<CategoryType> types = this.baseMapper.listMerchantGoodsCategoryTypes(merchantId);
        if (!CollectionUtils.isEmpty(types)) {
            types.sort(CategoryType::compareTo);
        }
        return types;
    }

    @Override
    public GoodsDetailDTO getGoodsDetail(@NonNull Long id) {
        Long userId = SessionContextHolder.getId();
        GoodsDetailDTO detail = GoodsDetailDTO.builder().build();
        // 获取商品信息
        Goods goods = ResourceUtils.required(this.getById(id), "商品不存在");
        //查询商品是否被收藏
        Integer count = this.baseMapper.goodsFavorite(id,userId);
        detail.setGoods(GoodsCopier.INSTANCE.copying(goods));
        if (count > 0){
            detail.getGoods().setFavorites(true);
        }
        // 获取用户可用优惠券
        if (!CollectionUtils.isEmpty(goods.getCouponIds())) {
            detail.setCoupons(this.couponService.listUserGoodsActiveCoupons(userId, goods.getCouponIds()));
        }

        // 获取商品评价列表
        CommentQueryParam param = CommentQueryParam.builder().size(10).build();
        detail.setComments(this.commentService.listGoodsComments(id, param));
        return detail;
    }

    @Override
    public List<GoodsDTO> listMerchantSimpleGoods(@NonNull Long merchantId, @NonNull GoodsQueryParam param) {
        return this.baseMapper.listMerchantSimpleGoods(merchantId, param);
    }



    @Override
    public Long addGoods(GoodsModifyParam param){
        Goods goods = GoodsCopier.INSTANCE.copying(param);
        goods.setScore(0D);
        goods.setSale(0);
        goods.setStatus(GoodsStatus.ONLINE);
        this.save(goods);
        return goods.getId();
    }

    @Override
    public void updateGoods(@NonNull Long id, @NonNull GoodsModifyParam param) {
        Goods goods = ResourceUtils.required(this.getById(id), "商品不存在");
        GoodsCopier.INSTANCE.copying(param, goods);
        this.updateById(goods);
    }

    @Override
    public void deleteGoods(@NonNull Long id) {
        ResourceUtils.required(this.getById(id), "商品不存在");
        this.baseMapper.deleteById(id);
    }

    @Override
    public Pagination<GoodsDTO> getGoodsPaging(@NonNull GoodsPagingParam param) {
        Page<Goods> page = this.lambdaQuery()
                .like(StringUtils.nonEmpty(param.getName()), Goods::getName, param.getName())
                .orderByAsc(Goods::getStatus).page(PagingUtils.convert(param.getPaging()));
        return PagingUtils.convert(page, GoodsCopier.INSTANCE::copying);
    }
}
