package com.chelvc.cloud.vehicle.server.service.impl;

import com.chelvc.cloud.vehicle.server.dao.HelpCategoryMapper;
import com.chelvc.cloud.vehicle.server.entity.HelpCategory;
import com.chelvc.cloud.vehicle.server.service.IHelpCategoryService;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * 帮助分类Service业务层处理
 * 
 * @author liude
 * @date 2023-11-08
 */
@Service
@RequiredArgsConstructor(onConstructor = @__({@Autowired, @Lazy}))
public class HelpCategoryServiceImpl implements IHelpCategoryService
{
    private final HelpCategoryMapper helpCategoryMapper;

    /**
     * 查询帮助分类
     * 
     * @param id 帮助分类主键
     * @return 帮助分类
     */
    @Override
    public HelpCategory selectHelpCategoryById(Long id)
    {
        return helpCategoryMapper.selectHelpCategoryById(id);
    }

    /**
     * 查询帮助分类列表
     * 
     * @param helpCategory 帮助分类
     * @return 帮助分类
     */
    @Override
    public List<HelpCategory> selectHelpCategoryList(HelpCategory helpCategory)
    {
        return helpCategoryMapper.selectHelpCategoryList(helpCategory);
    }

    /**
     * 新增帮助分类
     * 
     * @param helpCategory 帮助分类
     * @return 结果
     */
    @Override
    public int insertHelpCategory(HelpCategory helpCategory)
    {
        return helpCategoryMapper.insertHelpCategory(helpCategory);
    }

    /**
     * 修改帮助分类
     * 
     * @param helpCategory 帮助分类
     * @return 结果
     */
    @Override
    public int updateHelpCategory(HelpCategory helpCategory)
    {
        return helpCategoryMapper.updateHelpCategory(helpCategory);
    }

    /**
     * 批量删除帮助分类
     * 
     * @param ids 需要删除的帮助分类主键
     * @return 结果
     */
    @Override
    public int deleteHelpCategoryByIds(Long[] ids)
    {
        return helpCategoryMapper.deleteHelpCategoryByIds(ids);
    }

    /**
     * 删除帮助分类信息
     * 
     * @param id 帮助分类主键
     * @return 结果
     */
    @Override
    public int deleteHelpCategoryById(Long id)
    {
        return helpCategoryMapper.deleteHelpCategoryById(id);
    }
}
