package com.chelvc.cloud.vehicle.server.service.impl;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.chelvc.cloud.vehicle.api.dto.NoticeDTO;
import com.chelvc.cloud.vehicle.api.param.NoticeModifyParam;
import com.chelvc.cloud.vehicle.api.param.NoticePagingParam;
import com.chelvc.cloud.vehicle.server.copier.NoticeCopier;
import com.chelvc.cloud.vehicle.server.dao.NoticeMapper;
import com.chelvc.cloud.vehicle.server.entity.Notice;
import com.chelvc.cloud.vehicle.server.service.NoticeService;
import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.base.util.ResourceUtils;
import com.chelvc.framework.common.model.Pagination;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.database.util.PagingUtils;
import lombok.NonNull;
import lombok.RequiredArgsConstructor;
import org.apache.dubbo.config.annotation.DubboService;
import org.springframework.beans.factory.annotation.Autowired;

/**
 * 通知配置业务操作实现
 *
 * @author liude
 * @date 2023/12/2
 */
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
@DubboService(interfaceClass = com.chelvc.cloud.vehicle.api.service.NoticeService.class)
public class NoticeServiceImpl extends ServiceImpl<NoticeMapper, Notice> implements NoticeService,
        com.chelvc.cloud.vehicle.api.service.NoticeService {
    @Override
    public Long addNotice(NoticeModifyParam param){
        Notice notice = NoticeCopier.INSTANCE.copying(param);
        Long userId = SessionContextHolder.getId();
        notice.setCreator(userId.toString());
        notice.setUpdater(userId.toString());
        notice.setStatus("1");
        this.save(notice);
        return notice.getId();
    }

    @Override
    public void updateNotice(@NonNull Long id, @NonNull NoticeModifyParam param) {
        Notice notice = ResourceUtils.required(this.getById(id), "通知配置不存在");
        NoticeCopier.INSTANCE.copying(param, notice);
        this.updateById(notice);
    }

    @Override
    public Pagination<NoticeDTO> getNoticePaging(@NonNull NoticePagingParam param) {
        Page<Notice> page = this.lambdaQuery()
                .like(StringUtils.nonEmpty(param.getTitle()), Notice::getTitle, param.getTitle())
                .orderByAsc(Notice::getStatus).page(PagingUtils.convert(param.getPaging()));
        return PagingUtils.convert(page, NoticeCopier.INSTANCE::copying);
    }
}
