package com.chelvc.cloud.vehicle.server.service.impl;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.chelvc.cloud.vehicle.api.dto.CartPromotionItemDTO;
import com.chelvc.cloud.vehicle.api.dto.OmsCartItemDTO;
import com.chelvc.cloud.vehicle.api.param.OmsCartItemModifyParam;
import com.chelvc.cloud.vehicle.server.copier.CartItemCopier;
import com.chelvc.cloud.vehicle.server.dao.OmsCartItemMapper;
import com.chelvc.cloud.vehicle.server.entity.OmsCartItem;
import com.chelvc.cloud.vehicle.server.service.OmsCartItemService;
import com.chelvc.framework.base.context.SessionContextHolder;
import lombok.RequiredArgsConstructor;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang.ObjectUtils;
import org.apache.dubbo.config.annotation.DubboService;
import org.springframework.beans.factory.annotation.Autowired;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 购物车Service业务层处理
 * 
 * @author liude
 * @date 2023-11-08
 */
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
@DubboService(interfaceClass = com.chelvc.cloud.vehicle.api.service.OmsCartItemService.class)
public class OmsCartItemServiceImpl extends ServiceImpl<OmsCartItemMapper, OmsCartItem> implements OmsCartItemService,
        com.chelvc.cloud.vehicle.api.service.OmsCartItemService {
    @Override
    public int add(OmsCartItemModifyParam cartItem) {
        int count;
        Long userId = SessionContextHolder.getId();
        OmsCartItem omsCartItem = CartItemCopier.INSTANCE.copying(cartItem);
        omsCartItem.setUserId(userId);
        omsCartItem.setMemberNickname(userId.toString());
        omsCartItem.setDeleteStatus(0);
        OmsCartItem existCartItem = getCartItem(omsCartItem);
        if (existCartItem == null) {
            cartItem.setCreateDate(new Date());
            count = this.baseMapper.insert(omsCartItem);
        } else {
            cartItem.setModifyDate(new Date());
            existCartItem.setQuantity(existCartItem.getQuantity() + cartItem.getQuantity());
            count = this.baseMapper.updateById(existCartItem);
        }
        return count;
    }

    /**
     * 根据会员id,商品id和规格获取购物车中商品
     */
    private OmsCartItem getCartItem(OmsCartItem cartItem) {
        List<OmsCartItem> cartItemList = this.lambdaQuery()
                .eq(OmsCartItem::getUserId, cartItem.getUserId())
                .eq(OmsCartItem::getGoodsId, cartItem.getGoodsId()).list();
        if (!CollectionUtils.isEmpty(cartItemList)) {
            return cartItemList.get(0);
        }
        return null;
    }

    @Override
    public List<OmsCartItemDTO> listCartItem() {
        Long userId = SessionContextHolder.getId();
        OmsCartItem omsCartItem = new OmsCartItem();
        omsCartItem.setUserId(userId);
        List<OmsCartItem> cartItemList = this.lambdaQuery()
                .eq(OmsCartItem::getUserId, userId).list();
        List<OmsCartItemDTO> omsCartItemList = CartItemCopier.INSTANCE.copying(cartItemList);
        return omsCartItemList;
    }

    @Override
    public List<OmsCartItemDTO> listPromotion(List<Long> cartIds) {
        List<OmsCartItemDTO> cartItemList = listCartItem();
        if (!CollectionUtils.isEmpty(cartIds)) {
            cartItemList = cartItemList.stream().filter(item -> cartIds.contains(item.getId())).collect(Collectors.toList());
        }
        List<CartPromotionItemDTO> cartPromotionItemList = new ArrayList<>();
        if (!CollectionUtils.isEmpty(cartItemList)) {
//            cartPromotionItemList.add(cartItemList);
//            cartPromotionItemList = promotionService.calcCartPromotion(cartItemList);
        }
        return cartItemList;
    }

    @Override
    public int updateQuantity(Long id, Integer quantity) {
        Long userId = SessionContextHolder.getId();
        OmsCartItem cartItem = this.lambdaQuery()
                .eq(OmsCartItem::getUserId, userId).eq(
                        OmsCartItem::getGoodsId, id
                ).getEntity();
        cartItem.setQuantity(quantity);
        cartItem.setDeleteStatus(0);
        return this.baseMapper.updateById(cartItem);
    }

    @Override
    public void delete(List<Long> ids) {
        Long userId = SessionContextHolder.getId();
        for (Long id : ids) {
            OmsCartItem cartItem = this.lambdaQuery()
                    .eq(OmsCartItem::getUserId, userId).eq(
                            OmsCartItem::getGoodsId, id
                    ).getEntity();
            cartItem.setDeleteStatus(1);
            this.baseMapper.updateById(cartItem);
        }
    }

    @Override
    public void delete(List<Long> ids,Long userId) {
        for (Long id : ids) {
            OmsCartItem cartItem = this.lambdaQuery()
                    .eq(OmsCartItem::getUserId, userId).eq(
                            OmsCartItem::getId, id
                    ).one();
            if (cartItem != null){
                cartItem.setDeleteStatus(1);
                this.baseMapper.updateById(cartItem);
            }
        }
    }

//    @Override
//    public CartProduct getCartProduct(Long productId) {
//        return productDao.getCartProduct(productId);
//    }

    @Override
    public int updateAttr(OmsCartItemModifyParam cartItem) {
        //删除原购物车信息
        OmsCartItem updateCart = new OmsCartItem();
        updateCart.setId(cartItem.getId());
        updateCart.setUpdateTime(new Date());
        updateCart.setDeleteStatus(1);
        this.baseMapper.updateById(updateCart);
        cartItem.setId(null);
        add(cartItem);
        return 1;
    }

    @Override
    public void clear() {
        Long userId = SessionContextHolder.getId();
        List<OmsCartItem> cartItemList = this.lambdaQuery()
                .eq(OmsCartItem::getUserId, userId).list();
        for (OmsCartItem omsCartItem : cartItemList) {
            omsCartItem.setDeleteStatus(1);
            this.baseMapper.updateById(omsCartItem);
        }
    }
}
