package com.chelvc.cloud.vehicle.server.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.chelvc.cloud.vehicle.api.dto.OmsOrderOperateHistoryDTO;
import com.chelvc.cloud.vehicle.server.dao.OmsOrderOperateHistoryMapper;
import com.chelvc.cloud.vehicle.server.entity.OmsOrderOperateHistory;
import com.chelvc.cloud.vehicle.server.service.IOmsOrderOperateHistoryService;
import com.chelvc.framework.common.model.Pagination;
import com.chelvc.framework.common.model.Paging;
import com.chelvc.framework.common.util.BeanCopyUtil;
import com.chelvc.framework.database.util.PagingUtils;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;

/**
 * 订单操作历史记录Service业务层处理
 * 
 * @author liude
 * @date 2023-11-08
 */
@Service
public class OmsOrderOperateHistoryServiceImpl extends ServiceImpl<OmsOrderOperateHistoryMapper, OmsOrderOperateHistory>
        implements IOmsOrderOperateHistoryService
{
    /**
     * 查询订单操作历史记录
     * 
     * @param id 订单操作历史记录主键
     * @return 订单操作历史记录
     */
    @Override
    public OmsOrderOperateHistory selectOmsOrderOperateHistoryById(Long id)
    {
        return this.baseMapper.selectOmsOrderOperateHistoryById(id);
    }

    /**
     * 查询订单操作历史记录列表
     * 
     * @param omsOrderOperateHistory 订单操作历史记录
     * @return 订单操作历史记录
     */
    @Override
    public List<OmsOrderOperateHistory> selectOmsOrderOperateHistoryList(OmsOrderOperateHistory omsOrderOperateHistory)
    {
        return this.baseMapper.selectOmsOrderOperateHistoryList(omsOrderOperateHistory);
    }

    /**
     * 新增订单操作历史记录
     * 
     * @param omsOrderOperateHistory 订单操作历史记录
     * @return 结果
     */
    @Override
    public int insertOmsOrderOperateHistory(OmsOrderOperateHistory omsOrderOperateHistory)
    {
        omsOrderOperateHistory.setCreateTime(new Date());
        return this.baseMapper.insertOmsOrderOperateHistory(omsOrderOperateHistory);
    }

    /**
     * 修改订单操作历史记录
     * 
     * @param omsOrderOperateHistory 订单操作历史记录
     * @return 结果
     */
    @Override
    public int updateOmsOrderOperateHistory(OmsOrderOperateHistory omsOrderOperateHistory)
    {
        return this.baseMapper.updateOmsOrderOperateHistory(omsOrderOperateHistory);
    }

    /**
     * 批量删除订单操作历史记录
     * 
     * @param ids 需要删除的订单操作历史记录主键
     * @return 结果
     */
    @Override
    public int deleteOmsOrderOperateHistoryByIds(Long[] ids)
    {
        return this.baseMapper.deleteOmsOrderOperateHistoryByIds(ids);
    }

    /**
     * 删除订单操作历史记录信息
     * 
     * @param id 订单操作历史记录主键
     * @return 结果
     */
    @Override
    public int deleteOmsOrderOperateHistoryById(Long id)
    {
        return this.baseMapper.deleteOmsOrderOperateHistoryById(id);
    }

    /**
     * 根据用户查询订单通知信息
     *
     * @param userId 用户id
     * @return 订单操作历史记录
     */
    @Override
    public Pagination<OmsOrderOperateHistoryDTO> selectOmsOrderOperateHistoryListByUserId(Long userId, Paging paging) {
        Page<OmsOrderOperateHistory> page = this.lambdaQuery()
                .eq(OmsOrderOperateHistory::getUserId, userId)
                .orderByAsc(OmsOrderOperateHistory::getCreateTime).page(PagingUtils.convert(paging));
        List<OmsOrderOperateHistory> OmsOrderOperateHistories = page.getRecords();
        List<OmsOrderOperateHistoryDTO> copy = BeanCopyUtil.copy(OmsOrderOperateHistories, OmsOrderOperateHistoryDTO.class);
        return PagingUtils.convert(page, copy);
    }

    /**
     * 将新订单通知标识更改
     *
     * @param userId 用户id
     */
    @Override
    public void changeUserOrderInformFlag(Long userId) {
        this.baseMapper.changeUserOrderInformFlag(userId);
    }

    @Override
    public Integer queryNewInformNum(Long userId) {
        LambdaQueryWrapper<OmsOrderOperateHistory> lwq = Wrappers.lambdaQuery();
        lwq.eq(OmsOrderOperateHistory::getUserId, userId);
        lwq.eq(OmsOrderOperateHistory::getFlag, 0);
        return baseMapper.selectCount(lwq);
    }
}
