package com.chelvc.cloud.vehicle.server.service.impl;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.chelvc.cloud.vehicle.api.dto.CalcAmountDTO;
import com.chelvc.cloud.vehicle.api.dto.CartPromotionItemDTO;
import com.chelvc.cloud.vehicle.api.dto.ConfirmOrderResultDTO;
import com.chelvc.cloud.vehicle.api.dto.CouponDTO;
import com.chelvc.cloud.vehicle.api.dto.OmsCartItemDTO;
import com.chelvc.cloud.vehicle.api.dto.OmsOrderDTO;
import com.chelvc.cloud.vehicle.api.dto.OmsOrderDetailDTO;
import com.chelvc.cloud.vehicle.api.dto.OmsOrderExampleDTO;
import com.chelvc.cloud.vehicle.api.dto.OmsOrderItemDTO;
import com.chelvc.cloud.vehicle.api.dto.OmsOrderItemExampleDTO;
import com.chelvc.cloud.vehicle.api.dto.UserCouponDTO;
import com.chelvc.cloud.vehicle.api.param.OmsOrderModifyParam;
import com.chelvc.cloud.vehicle.api.param.OrderPagingParam;
import com.chelvc.cloud.vehicle.api.service.UserCouponService;
import com.chelvc.cloud.vehicle.server.copier.OrderCopier;
import com.chelvc.cloud.vehicle.server.dao.OmsOrderItemMapper;
import com.chelvc.cloud.vehicle.server.dao.OmsOrderMapper;
import com.chelvc.cloud.vehicle.server.dao.OmsOrderSettingMapper;
import com.chelvc.cloud.vehicle.server.entity.OmsOrder;
import com.chelvc.cloud.vehicle.server.entity.OmsOrderItem;
import com.chelvc.cloud.vehicle.server.entity.OmsOrderSetting;
import com.chelvc.cloud.vehicle.server.service.IOmsOrderItemService;
import com.chelvc.cloud.vehicle.server.service.IOmsOrderService;
import com.chelvc.cloud.vehicle.server.service.OmsCartItemService;
import com.chelvc.cloud.vehicle.server.service.OmsOrderSettingService;
import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.common.model.Pagination;
import com.chelvc.framework.database.util.PagingUtils;
import lombok.RequiredArgsConstructor;
import org.apache.dubbo.config.annotation.DubboService;
import org.apache.http.util.Asserts;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;

/**
 * 订单Service业务层处理
 * 
 * @author liude
 * @date 2023-11-08
 */
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
@DubboService(interfaceClass = com.chelvc.cloud.vehicle.api.service.IOmsOrderService.class)
public class OmsOrderServiceImpl extends ServiceImpl<OmsOrderMapper, OmsOrder> implements IOmsOrderService,
        com.chelvc.cloud.vehicle.api.service.IOmsOrderService {
    private final OmsCartItemService omsCartItemService;

    private final OmsOrderSettingService orderSettingService;

    private OmsOrderSettingMapper omsOrderSettingMapper;

    private final IOmsOrderItemService omsOrderItemService;

    private final UserCouponService userCouponService;

    @Override
    public ConfirmOrderResultDTO generateConfirmOrder(OmsOrderModifyParam orderParam) {
        ConfirmOrderResultDTO result = new ConfirmOrderResultDTO();
        //获取购物车信息
        Long userId = SessionContextHolder.getId();
        List<OmsCartItemDTO> cartItemList = omsCartItemService.listPromotion(orderParam.getCartIds());
        result.setCartItemList(cartItemList);
//        //获取用户收货地址列表
//        List<UmsMemberReceiveAddress> memberReceiveAddressList = memberReceiveAddressService.list();
//        result.setMemberReceiveAddressList(memberReceiveAddressList);
        //获取用户可用优惠券列表
//        List<SmsCouponHistoryDetail> couponHistoryDetailList = memberCouponService.listCart(cartPromotionItemList, 1);
//        result.setCouponHistoryDetailList(couponHistoryDetailList);
//        //获取用户积分
//        result.setMemberIntegration(currentMember.getIntegration());
//        //获取积分使用规则
//        UmsIntegrationConsumeSetting integrationConsumeSetting = integrationConsumeSettingMapper.selectByPrimaryKey(1L);
//        result.setIntegrationConsumeSetting(integrationConsumeSetting);
//        //计算总金额、活动优惠、应付金额
        CalcAmountDTO calcAmount = calcCartAmount(cartItemList);
        result.setCalcAmount(calcAmount);
        return result;
    }

    @Override
    public Map<String, Object> generateOrder(OmsOrderModifyParam orderParam) {
        List<OmsOrderItem> orderItemList = new ArrayList<>();
        //获取购物车及优惠信息
        Long userId = SessionContextHolder.getId();
        List<OmsCartItemDTO> cartPromotionItemList = omsCartItemService.listPromotion(orderParam.getCartIds());
        for (OmsCartItemDTO cartPromotionItem : cartPromotionItemList) {
            //生成下单商品信息
            OmsOrderItem orderItem = new OmsOrderItem();
            orderItem.setGoodsId(cartPromotionItem.getGoodsId());
            orderItem.setGoodsName(cartPromotionItem.getGoodsName());
            orderItem.setGoodsPic(cartPromotionItem.getGoodsPic());
            orderItem.setGoodsAttr(cartPromotionItem.getGoodsAttr());
            orderItem.setGoodsBrand(cartPromotionItem.getGoodsBrand());
            orderItem.setGoodsSn(cartPromotionItem.getGoodsSn());
            orderItem.setGoodsPrice(cartPromotionItem.getPrice());
            orderItem.setGoodsQuantity(cartPromotionItem.getQuantity());
            orderItem.setGoodsSkuId(cartPromotionItem.getGoodsSkuId());
            orderItem.setGoodsSkuCode(cartPromotionItem.getGoodsSkuCode());
            orderItem.setGoodsCategoryId(cartPromotionItem.getGoodsCategoryId());
//            orderItem.setPromotionAmount(cartPromotionItem.getReduceAmount());
//            orderItem.setPromotionName(cartPromotionItem.getPromotionMessage());
            orderItem.setGiftIntegration(0L);
            orderItem.setGiftGrowth(0);
            orderItemList.add(orderItem);
        }
        //判断购物车中商品是否都有库存
//        if (!hasStock(cartPromotionItemList)) {
//            Asserts.fail("库存不足，无法下单");
//        }
        //判断使用使用了优惠券
        if (orderParam.getCouponId() == null) {
            //不用优惠券
            for (OmsOrderItem orderItem : orderItemList) {
                orderItem.setCouponAmount(new BigDecimal(0));
            }
        } else {
            //使用优惠券
            UserCouponDTO couponHistoryDetail = getUseCoupon(cartPromotionItemList, orderParam.getCouponId());
            if (couponHistoryDetail == null) {
                Asserts.check(true,"该优惠券不可用");
            }
            //对下单商品的优惠券进行处理
//            handleCouponAmount(orderItemList, couponHistoryDetail);
        }
        //判断是否使用积分
        if (orderParam.getUseIntegration() == null||orderParam.getUseIntegration().equals(0)) {
            //不使用积分
            for (OmsOrderItem orderItem : orderItemList) {
                orderItem.setIntegrationAmount(new BigDecimal(0));
            }
        } else {
            //使用积分
            BigDecimal totalAmount = calcTotalAmount(orderItemList);
//            BigDecimal integrationAmount = getUseIntegrationAmount(orderParam.getUseIntegration(), totalAmount, currentMember, orderParam.getCouponId() != null);
//            if (integrationAmount.compareTo(new BigDecimal(0)) == 0) {
//                Asserts.check(true,"积分不可用");
//            } else {
                //可用情况下分摊到可用商品中
//                for (OmsOrderItemDTO orderItem : orderItemList) {
//                    BigDecimal perAmount = orderItem.getProductPrice().divide(totalAmount, 3, RoundingMode.HALF_EVEN).multiply(integrationAmount);
//                    orderItem.setIntegrationAmount(perAmount);
////                }
//            }
        }
        //计算order_item的实付金额
//        handleRealAmount(orderItemList);
        //进行库存锁定
//        lockStock(cartPromotionItemList);
        //根据商品合计、运费、活动优惠、优惠券、积分计算应付金额
        OmsOrder order = new OmsOrder();
        order.setDiscountAmount(new BigDecimal(0));
        order.setTotalAmount(calcTotalAmount(orderItemList));
        order.setFreightAmount(new BigDecimal(0));
        order.setPromotionAmount(calcPromotionAmount(orderItemList));
        order.setPromotionInfo(getOrderPromotionInfo(orderItemList));
        if (orderParam.getCouponId() == null) {
            order.setCouponAmount(new BigDecimal(0));
        } else {
            order.setCouponId(orderParam.getCouponId());
            order.setCouponAmount(calcCouponAmount(orderItemList));
        }
        if (orderParam.getUseIntegration() == null) {
            order.setIntegration(0);
            order.setIntegrationAmount(new BigDecimal(0));
        } else {
            order.setIntegration(orderParam.getUseIntegration());
            order.setIntegrationAmount(calcIntegrationAmount(orderItemList));
        }
        order.setPayAmount(calcPayAmount(order));
        //转化为订单信息并插入数据库
        order.setUserId(userId);
        order.setCreateTime(new Date());
        order.setMemberUsername(userId.toString());
        //支付方式：0->未支付；1->支付宝；2->微信
        order.setPayType(orderParam.getPayType());
        //订单来源：0->PC订单；1->app订单
        order.setSourceType(1);
        //订单状态：0->待付款；1->待发货；2->已发货；3->已完成；4->已关闭；5->无效订单
        order.setStatus(0);
        //订单类型：0->正常订单；1->秒杀订单
        order.setOrderType(0);
        //收货人信息：姓名、电话、邮编、地址
//        UmsMemberReceiveAddress address = memberReceiveAddressService.getItem(orderParam.getMemberReceiveAddressId());
//        order.setReceiverName(address.getName());
//        order.setReceiverPhone(address.getPhoneNumber());
//        order.setReceiverPostCode(address.getPostCode());
//        order.setReceiverProvince(address.getProvince());
//        order.setReceiverCity(address.getCity());
//        order.setReceiverRegion(address.getRegion());
//        order.setReceiverDetailAddress(address.getDetailAddress());
        //0->未确认；1->已确认
        order.setConfirmStatus(0);
        order.setDeleteStatus(0);
        //计算赠送积分
        order.setIntegration(calcGifIntegration(orderItemList));
        //计算赠送成长值
        order.setGrowth(0);
        //生成订单号
        order.setOrderSn(generateOrderSn(order));
        //设置自动收货天数
        List<OmsOrderSetting> orderSettings = orderSettingService.listOrderSetting();
        if(!CollectionUtils.isEmpty(orderSettings)){
            order.setAutoConfirmDay(orderSettings.get(0).getConfirmOvertime());
        }
        //插入order表和order_item表
        this.baseMapper.insert(order);
        for (OmsOrderItem orderItem : orderItemList) {
            orderItem.setOrderId(order.getId());
            orderItem.setOrderSn(order.getOrderSn());
        }
        omsOrderItemService.add(orderItemList);
        //如使用优惠券更新优惠券使用状态
        if (orderParam.getCouponId() != null) {
//            updateCouponStatus(orderParam.getCouponId(), userId, 1);
        }
        //如使用积分需要扣除积分
//        if (orderParam.getUseIntegration() != null) {
//            order.setUseIntegration(orderParam.getUseIntegration());
//            memberService.updateIntegration(currentMember.getId(), currentMember.getIntegration() - orderParam.getUseIntegration());
//        }
        //删除购物车中的下单商品
        deleteCartItemList(cartPromotionItemList, userId);
        //发送延迟消息取消订单
        sendDelayMessageCancelOrder(order.getId());
        Map<String, Object> result = new HashMap<>();
        result.put("order", order);
        result.put("orderItemList", orderItemList);
        return result;
    }

    @Override
    public Integer paySuccess(Long orderId, Integer payType) {
        //修改订单支付状态
        OmsOrder order = this.baseMapper.selectById(orderId);
        order.setStatus(1);
        order.setPaymentTime(new Date());
        order.setPayType(payType);
        this.baseMapper.updateById(order);
        //恢复所有下单商品的锁定库存，扣减真实库存
        OmsOrderDetailDTO orderDetail = this.baseMapper.getDetail(orderId);
        int count = this.baseMapper.updateSkuStock(orderDetail.getOrderItemList());
        return count;
    }

    @Override
    public Integer cancelTimeOutOrder() {
        Integer count=0;
        //查询订单设置
        List<OmsOrderSetting> orderSettings = orderSettingService.listOrderSetting();
        if(!CollectionUtils.isEmpty(orderSettings)){
            Asserts.check(true,"未设置订单超时设置！");
        }
        //查询超时、未支付的订单及订单详情
        List<OmsOrderDetailDTO> timeOutOrders = this.baseMapper.getTimeOutOrders(orderSettings.get(0).getNormalOrderOvertime());
        if (CollectionUtils.isEmpty(timeOutOrders)) {
            return count;
        }
        //修改订单状态为交易取消
        List<Long> ids = new ArrayList<>();
        for (OmsOrderDetailDTO timeOutOrder : timeOutOrders) {
            ids.add(timeOutOrder.getId());
        }
        this.baseMapper.updateOrderStatus(ids, 4);
        for (OmsOrderDetailDTO timeOutOrder : timeOutOrders) {
            //解除订单商品库存锁定
//            omsOrderMapper.releaseSkuStockLock(timeOutOrder.getOrderItemList());
            //修改优惠券使用状态
//            updateCouponStatus(timeOutOrder.getCouponId(), timeOutOrder.getUserId(), 0);
            //返还使用积分
//            if (timeOutOrder.getUseIntegration() != null) {
//                UmsMember member = memberService.getById(timeOutOrder.getMemberId());
//                memberService.updateIntegration(timeOutOrder.getMemberId(), member.getIntegration() + timeOutOrder.getUseIntegration());
//            }
        }
        return timeOutOrders.size();
    }

    @Override
    public void cancelOrder(Long orderId) {
        //查询未付款的取消订单
        OmsOrderExampleDTO example = new OmsOrderExampleDTO();
        example.createCriteria().andIdEqualTo(orderId).andStatusEqualTo(0).andDeleteStatusEqualTo(0);
        List<OmsOrder> cancelOrderList = this.baseMapper.selectByExample(example);
        if (CollectionUtils.isEmpty(cancelOrderList)) {
            return;
        }
        OmsOrder cancelOrder = cancelOrderList.get(0);
        if (cancelOrder != null) {
            //修改订单状态为取消
            cancelOrder.setStatus(4);
//            this.baseMapper.updateByPrimaryKeySelective(cancelOrder);
            OmsOrderItem orderItem = new OmsOrderItem();
            orderItem.setOrderId(orderId);
            List<OmsOrderItem> orderItemList = this.omsOrderItemService.selectByExample(orderItem);
            //解除订单商品库存锁定
            if (!CollectionUtils.isEmpty(orderItemList)) {
                this.baseMapper.releaseSkuStockLock(orderItemList);
            }
            //修改优惠券使用状态
//            updateCouponStatus(cancelOrder.getCouponId(), cancelOrder.getUserId(), 0);
            //返还使用积分
//            if (cancelOrder.getUseIntegration() != null) {
//                UmsMember member = memberService.getById(cancelOrder.getMemberId());
//                memberService.updateIntegration(cancelOrder.getMemberId(), member.getIntegration() + cancelOrder.getUseIntegration());
//            }
        }
    }

    @Override
    public void sendDelayMessageCancelOrder(Long orderId) {
        //获取订单超时时间
//        OmsOrderSetting orderSetting = orderSettingService.selectByPrimaryKey(1L);
//        long delayTimes = orderSetting.getNormalOrderOvertime() * 60 * 1000;
        //发送延迟消息
//        cancelOrderSender.sendMessage(orderId, delayTimes);
    }

    @Override
    public void confirmReceiveOrder(Long orderId) {
        Long userId = SessionContextHolder.getId();
        OmsOrder order = this.baseMapper.selectOmsOrderById(orderId);
        if(!userId.equals(order.getUserId())){
            Asserts.check(true,"不能确认他人订单！");
        }
        if(order.getStatus()!=2){
            Asserts.check(true,"该订单还未发货！");
        }
        order.setStatus(3);
        order.setConfirmStatus(1);
        order.setReceiveTime(new Date());
        this.baseMapper.updateById(order);
    }
    @Override
    public Pagination<OmsOrderDTO> getOrderPaging(OrderPagingParam param){
        Page<OmsOrder> page = this.lambdaQuery()
                .eq(OmsOrder :: getStatus,param.getStatus())
                .orderByDesc(OmsOrder :: getCreateTime)
                .page(PagingUtils.convert(param.getPaging()));
       return PagingUtils.convert(page, OrderCopier.INSTANCE::copying);
    }

    @Override
    public Pagination<OmsOrderDTO> getOmsOrderPaging(OrderPagingParam param){
        Page<OmsOrder> page = this.lambdaQuery()
                .eq(OmsOrder :: getStatus,param.getStatus())
                .orderByDesc(OmsOrder :: getCreateTime)
                .page(PagingUtils.convert(param.getPaging()));
        return PagingUtils.convert(page, OrderCopier.INSTANCE::copying);
    }

    @Override
    public OmsOrderDetailDTO detail(Long orderId) {
        OmsOrder omsOrder = this.baseMapper.selectOmsOrderById(orderId);
        OmsOrderItem orderItem = new OmsOrderItem();
        orderItem.setOrderId(orderId);
        List<OmsOrderItem> orderItemList = omsOrderItemService.selectByExample(orderItem);
        OmsOrderDetailDTO orderDetail = new OmsOrderDetailDTO();
        BeanUtils.copyProperties(omsOrder,orderDetail);
//        orderDetail.setOrderItemList(orderItemList);
        return orderDetail;
    }

    @Override
    public void deleteOrder(Long orderId) {
        Long userId = SessionContextHolder.getId();
        OmsOrder order = this.baseMapper.selectOmsOrderById(orderId);
        if(!userId.equals(order.getUserId())){
            Asserts.check(true,"不能删除他人订单！");
        }
        if(order.getStatus()==3||order.getStatus()==4){
            order.setDeleteStatus(1);
            this.baseMapper.updateById(order);
        }else{
            Asserts.check(true,"只能删除已完成或已关闭的订单！");
        }
    }

    /**
     * 生成18位订单编号:8位日期+2位平台号码+2位支付方式+6位以上自增id
     */
    private String generateOrderSn(OmsOrder order) {
        StringBuilder sb = new StringBuilder();
        String date = new SimpleDateFormat("yyyyMMdd").format(new Date());
//        String key = REDIS_DATABASE+":"+ REDIS_KEY_ORDER_ID + date;
//        Long increment = redisService.incr(key, 1);
        sb.append(date);
        sb.append(String.format("%02d", order.getSourceType()));
        sb.append(String.format("%02d", order.getPayType()));
//        String incrementStr = increment.toString();
//        if (incrementStr.length() <= 6) {
//            sb.append(String.format("%06d", increment));
//        } else {
//            sb.append(incrementStr);
//        }
        return sb.toString();
    }

    /**
     * 删除下单商品的购物车信息
     */
    private void deleteCartItemList(List<OmsCartItemDTO> cartPromotionItemList, Long userId) {
        List<Long> ids = new ArrayList<>();
        for (OmsCartItemDTO cartPromotionItem : cartPromotionItemList) {
            ids.add(cartPromotionItem.getId());
        }
        omsCartItemService.delete(ids,userId);
    }

    /**
     * 计算该订单赠送的成长值
     */
    private Integer calcGiftGrowth(List<OmsOrderItemDTO> orderItemList) {
        Integer sum = 0;
        for (OmsOrderItemDTO orderItem : orderItemList) {
            sum = sum + orderItem.getGiftGrowth() * orderItem.getGoodsQuantity();
        }
        return sum;
    }

    /**
     * 计算该订单赠送的积分
     */
    private Integer calcGifIntegration(List<OmsOrderItem> orderItemList) {
        int sum = 0;
        for (OmsOrderItem orderItem : orderItemList) {
            sum += orderItem.getGiftIntegration() * orderItem.getGoodsQuantity();
        }
        return sum;
    }

//    /**
//     * 将优惠券信息更改为指定状态
//     *
//     * @param couponId  优惠券id
//     * @param memberId  会员id
//     * @param useStatus 0->未使用；1->已使用
//     */
//    private void updateCouponStatus(Long couponId, Long memberId, Integer useStatus) {
//        if (couponId == null) return;
//        //查询第一张优惠券
//        UserCouponDTO example = new omsCartItemService();
//        example.createCriteria().andMemberIdEqualTo(memberId)
//                .andCouponIdEqualTo(couponId).andUseStatusEqualTo(useStatus == 0 ? 1 : 0);
//        List<SmsCouponHistory> couponHistoryList = couponHistoryMapper.selectByExample(example);
//        if (!CollectionUtils.isEmpty(couponHistoryList)) {
//            SmsCouponHistory couponHistory = couponHistoryList.get(0);
//            couponHistory.setUseTime(new Date());
//            couponHistory.setUseStatus(useStatus);
//            couponHistoryMapper.updateByPrimaryKeySelective(couponHistory);
//        }
//    }

    private void handleRealAmount(List<OmsOrderItemDTO> orderItemList) {
        for (OmsOrderItemDTO orderItem : orderItemList) {
            //原价-促销优惠-优惠券抵扣-积分抵扣
            BigDecimal realAmount = orderItem.getGoodsPrice()
                    .subtract(orderItem.getPromotionAmount())
                    .subtract(orderItem.getCouponAmount())
                    .subtract(orderItem.getIntegrationAmount());
            orderItem.setRealAmount(realAmount);
        }
    }

    /**
     * 获取订单促销信息
     */
    private String getOrderPromotionInfo(List<OmsOrderItem> orderItemList) {
        StringBuilder sb = new StringBuilder();
        for (OmsOrderItem orderItem : orderItemList) {
            sb.append(orderItem.getPromotionName());
            sb.append(";");
        }
        String result = sb.toString();
        if (result.endsWith(";")) {
            result = result.substring(0, result.length() - 1);
        }
        return result;
    }

    /**
     * 计算订单应付金额
     */
    private BigDecimal calcPayAmount(OmsOrder order) {
        //总金额+运费-促销优惠-优惠券优惠-积分抵扣
        BigDecimal payAmount = order.getTotalAmount()
                .add(order.getFreightAmount())
                .subtract(order.getPromotionAmount())
                .subtract(order.getCouponAmount())
                .subtract(order.getIntegrationAmount());
        return payAmount;
    }

    /**
     * 计算订单优惠券金额
     */
    private BigDecimal calcIntegrationAmount(List<OmsOrderItem> orderItemList) {
        BigDecimal integrationAmount = new BigDecimal(0);
        for (OmsOrderItem orderItem : orderItemList) {
            if (orderItem.getIntegrationAmount() != null) {
                integrationAmount = integrationAmount.add(orderItem.getIntegrationAmount().multiply(new BigDecimal(orderItem.getGoodsQuantity())));
            }
        }
        return integrationAmount;
    }

    /**
     * 计算订单优惠券金额
     */
    private BigDecimal calcCouponAmount(List<OmsOrderItem> orderItemList) {
        BigDecimal couponAmount = new BigDecimal(0);
        for (OmsOrderItem orderItem : orderItemList) {
            if (orderItem.getCouponAmount() != null) {
                couponAmount = couponAmount.add(orderItem.getCouponAmount().multiply(new BigDecimal(orderItem.getGoodsQuantity())));
            }
        }
        return couponAmount;
    }

    /**
     * 计算订单活动优惠
     */
    private BigDecimal calcPromotionAmount(List<OmsOrderItem> orderItemList) {
        BigDecimal promotionAmount = new BigDecimal(0);
        for (OmsOrderItem orderItem : orderItemList) {
            if (orderItem.getPromotionAmount() != null) {
                promotionAmount = promotionAmount.add(orderItem.getPromotionAmount().multiply(new BigDecimal(orderItem.getGoodsQuantity())));
            }
        }
        return promotionAmount;
    }

    /**
     * 获取可用积分抵扣金额
     *
     * @param useIntegration 使用的积分数量
     * @param totalAmount    订单总金额
     * @param currentMember  使用的用户
     * @param hasCoupon      是否已经使用优惠券
     */
//    private BigDecimal getUseIntegrationAmount(Integer useIntegration, BigDecimal totalAmount, UmsMember currentMember, boolean hasCoupon) {
//        BigDecimal zeroAmount = new BigDecimal(0);
//        //判断用户是否有这么多积分
//        if (useIntegration.compareTo(currentMember.getIntegration()) > 0) {
//            return zeroAmount;
//        }
//        //根据积分使用规则判断是否可用
//        //是否可与优惠券共用
//        UmsIntegrationConsumeSetting integrationConsumeSetting = integrationConsumeSettingMapper.selectByPrimaryKey(1L);
//        if (hasCoupon && integrationConsumeSetting.getCouponStatus().equals(0)) {
//            //不可与优惠券共用
//            return zeroAmount;
//        }
//        //是否达到最低使用积分门槛
//        if (useIntegration.compareTo(integrationConsumeSetting.getUseUnit()) < 0) {
//            return zeroAmount;
//        }
//        //是否超过订单抵用最高百分比
//        BigDecimal integrationAmount = new BigDecimal(useIntegration).divide(new BigDecimal(integrationConsumeSetting.getUseUnit()), 2, RoundingMode.HALF_EVEN);
//        BigDecimal maxPercent = new BigDecimal(integrationConsumeSetting.getMaxPercentPerOrder()).divide(new BigDecimal(100), 2, RoundingMode.HALF_EVEN);
//        if (integrationAmount.compareTo(totalAmount.multiply(maxPercent)) > 0) {
//            return zeroAmount;
//        }
//        return integrationAmount;
//    }

    /**
     * 对优惠券优惠进行处理
     *
     * @param orderItemList       order_item列表
     * @param couponHistoryDetail 可用优惠券详情
     */
//    private void handleCouponAmount(List<OmsOrderItemDTO> orderItemList, UserCouponDTO couponHistoryDetail) {
//        CouponDTO coupon = couponHistoryDetail.getCoupon();
//        if (coupon.getUseType().equals(0)) {
//            //全场通用
//            calcPerCouponAmount(orderItemList, coupon);
//        } else if (coupon.getUseType().equals(1)) {
//            //指定分类
//            List<OmsOrderItemDTO> couponOrderItemList = getCouponOrderItemByRelation(couponHistoryDetail, orderItemList, 0);
//            calcPerCouponAmount(couponOrderItemList, coupon);
//        } else if (coupon.getUseType().equals(2)) {
//            //指定商品
//            List<OmsOrderItemDTO> couponOrderItemList = getCouponOrderItemByRelation(couponHistoryDetail, orderItemList, 1);
//            calcPerCouponAmount(couponOrderItemList, coupon);
//        }
//    }

    /**
     * 对每个下单商品进行优惠券金额分摊的计算
     *
     * @param orderItemList 可用优惠券的下单商品商品
     */
    private void calcPerCouponAmount(List<OmsOrderItem> orderItemList, CouponDTO coupon) {
        BigDecimal totalAmount = calcTotalAmount(orderItemList);
        for (OmsOrderItem orderItem : orderItemList) {
            //(商品价格/可用商品总价)*优惠券面额
            BigDecimal couponAmount = orderItem.getGoodsPrice().divide(totalAmount, 3, RoundingMode.HALF_EVEN).multiply(new BigDecimal(coupon.getAmount()));
            orderItem.setCouponAmount(couponAmount);
        }
    }

    /**
     * 获取与优惠券有关系的下单商品
     *
     * @param couponHistoryDetail 优惠券详情
     * @param orderItemList       下单商品
     * @param type                使用关系类型：0->相关分类；1->指定商品
     */
//    private List<OmsOrderItemDTO> getCouponOrderItemByRelation(UserCouponDTO couponHistoryDetail, List<OmsOrderItemDTO> orderItemList, int type) {
//        List<OmsOrderItemDTO> result = new ArrayList<>();
//        if (type == 0) {
//            List<Long> categoryIdList = new ArrayList<>();
//            for (SmsCouponProductCategoryRelation productCategoryRelation : couponHistoryDetail.getCategoryRelationList()) {
//                categoryIdList.add(productCategoryRelation.getProductCategoryId());
//            }
//            for (OmsOrderItemDTO orderItem : orderItemList) {
//                if (categoryIdList.contains(orderItem.getProductCategoryId())) {
//                    result.add(orderItem);
//                } else {
//                    orderItem.setCouponAmount(new BigDecimal(0));
//                }
//            }
//        } else if (type == 1) {
//            List<Long> productIdList = new ArrayList<>();
//            for (SmsCouponProductRelation productRelation : couponHistoryDetail.getProductRelationList()) {
//                productIdList.add(productRelation.getProductId());
//            }
//            for (OmsOrderItemDTO orderItem : orderItemList) {
//                if (productIdList.contains(orderItem.getProductId())) {
//                    result.add(orderItem);
//                } else {
//                    orderItem.setCouponAmount(new BigDecimal(0));
//                }
//            }
//        }
//        return result;
//    }

    /**
     * 获取该用户可以使用的优惠券
     *
     * @param cartPromotionItemList 购物车优惠列表
     * @param couponId              使用优惠券id
     */
    private UserCouponDTO getUseCoupon(List<OmsCartItemDTO> cartPromotionItemList, Long couponId) {
//        List<UserCouponDTO> couponHistoryDetailList = memberCouponService.listCart(cartPromotionItemList, 1);
        List<UserCouponDTO> couponHistoryDetailList = new ArrayList<>();
        for (UserCouponDTO couponHistoryDetail : couponHistoryDetailList) {
            if (couponHistoryDetail.getId().equals(couponId)) {
                return couponHistoryDetail;
            }
        }
        return null;
    }

    /**
     * 计算总金额
     */
    private BigDecimal calcTotalAmount(List<OmsOrderItem> orderItemList) {
        BigDecimal totalAmount = new BigDecimal("0");
        for (OmsOrderItem item : orderItemList) {
            totalAmount = totalAmount.add(item.getGoodsPrice().multiply(new BigDecimal(item.getGoodsQuantity())));
        }
        return totalAmount;
    }

    /**
     * 锁定下单商品的所有库存
     */
//    private void lockStock(List<CartPromotionItemDTO> cartPromotionItemList) {
//        for (CartPromotionItemDTO cartPromotionItem : cartPromotionItemList) {
//            PmsSkuStock skuStock = skuStockMapper.selectByPrimaryKey(cartPromotionItem.getGoodsSkuId());
//            skuStock.setLockStock(skuStock.getLockStock() + cartPromotionItem.getQuantity());
//            skuStockMapper.updateByPrimaryKeySelective(skuStock);
//        }
//    }

    /**
     * 判断下单商品是否都有库存
     */
    private boolean hasStock(List<CartPromotionItemDTO> cartPromotionItemList) {
        for (CartPromotionItemDTO cartPromotionItem : cartPromotionItemList) {
            if (cartPromotionItem.getRealStock()==null||cartPromotionItem.getRealStock() <= 0) {
                return false;
            }
        }
        return true;
    }

    /**
     * 计算购物车中商品的价格
     */
    private CalcAmountDTO calcCartAmount(List<OmsCartItemDTO> cartPromotionItemList) {
       CalcAmountDTO calcAmount = new CalcAmountDTO();
        calcAmount.setFreightAmount(new BigDecimal(0));
        BigDecimal totalAmount = new BigDecimal("0");
        BigDecimal promotionAmount = new BigDecimal("0");
        for (OmsCartItemDTO cartPromotionItem : cartPromotionItemList) {
            totalAmount = totalAmount.add(cartPromotionItem.getPrice().multiply(new BigDecimal(cartPromotionItem.getQuantity())));
//            promotionAmount = promotionAmount.add(cartPromotionItem.getReduceAmount().multiply(new BigDecimal(cartPromotionItem.getQuantity())));
        }
        calcAmount.setTotalAmount(totalAmount);
        calcAmount.setPromotionAmount(promotionAmount);
        calcAmount.setPayAmount(totalAmount.subtract(promotionAmount));
        return calcAmount;
    }






}
