package com.chelvc.cloud.vehicle.server.service.impl;

import java.util.ArrayList;
import java.util.List;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.chelvc.cloud.vehicle.api.dto.OmsOrderSettingDTO;
import com.chelvc.cloud.vehicle.api.param.OmsOrderSettingModifyParam;
import com.chelvc.cloud.vehicle.server.copier.OrderSettingCopier;
import com.chelvc.cloud.vehicle.server.dao.OmsOrderSettingMapper;
import com.chelvc.cloud.vehicle.server.entity.OmsOrderSetting;
import com.chelvc.cloud.vehicle.server.service.OmsOrderSettingService;
import lombok.NonNull;
import lombok.RequiredArgsConstructor;
import org.apache.dubbo.config.annotation.DubboService;
import org.springframework.beans.factory.annotation.Autowired;

/**
 * 订单设置Service业务层处理
 * 
 * @author liude
 * @date 2023-11-08
 */
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
@DubboService(interfaceClass = com.chelvc.cloud.vehicle.api.service.OmsOrderSettingService.class)
public class OmsOrderSettingServiceImpl extends ServiceImpl<OmsOrderSettingMapper, OmsOrderSetting> implements
        OmsOrderSettingService,
        com.chelvc.cloud.vehicle.api.service.OmsOrderSettingService {
    /**
     * 查询订单设置
     *
     * @param id 订单设置主键
     * @return 订单设置
     */
    @Override
    public OmsOrderSettingDTO selectOmsOrderSettingById(@NonNull Long id)
    {
        OmsOrderSetting omsOrderSetting =this.baseMapper .selectOmsOrderSettingById(id);
        return OrderSettingCopier.INSTANCE.copying(omsOrderSetting);
    }

    /**
     * 查询订单设置列表
     *
     * @param param 订单设置
     * @return 订单设置
     */
    @Override
    public List<OmsOrderSettingDTO> selectOmsOrderSettingList(@NonNull OmsOrderSettingModifyParam param)
    {
        List<OmsOrderSettingDTO> omsOrderSettingDTOList = new ArrayList<>();
        OmsOrderSetting omsOrderSetting = OrderSettingCopier.INSTANCE.copying(param);
        List<OmsOrderSetting> omsOrderSettings = this.baseMapper.selectOmsOrderSettingList(omsOrderSetting);
        omsOrderSettings.forEach(item ->{
            omsOrderSettingDTOList.add(OrderSettingCopier.INSTANCE.copying(item));
        });
        return omsOrderSettingDTOList;
    }

    /**
     * 新增订单设置
     *
     * @param param 订单设置
     * @return 结果
     */
    @Override
    public int insertOmsOrderSetting(@NonNull OmsOrderSettingModifyParam param)
    {
        OmsOrderSetting omsOrderSetting = OrderSettingCopier.INSTANCE.copying(param);
        return this.baseMapper.insertOmsOrderSetting(omsOrderSetting);
    }

    /**
     * 修改订单设置
     *
     * @param param 订单设置
     * @return 结果
     */
    @Override
    public int updateOmsOrderSetting(@NonNull OmsOrderSettingModifyParam param)
    {
        OmsOrderSetting omsOrderSetting = OrderSettingCopier.INSTANCE.copying(param);
        return this.baseMapper.updateOmsOrderSetting(omsOrderSetting);
    }

    /**
     * 批量删除订单设置
     *
     * @param ids 需要删除的订单设置主键
     * @return 结果
     */
    @Override
    public int deleteOmsOrderSettingByIds(@NonNull Long[] ids)
    {
        return this.baseMapper.deleteOmsOrderSettingByIds(ids);
    }

    /**
     * 删除订单设置信息
     *
     * @param id 订单设置主键
     * @return 结果
     */
    @Override
    public int deleteOmsOrderSettingById(@NonNull Long id)
    {
        return this.baseMapper.deleteOmsOrderSettingById(id);
    }
    @Override
    public List<OmsOrderSetting> listOrderSetting(){
      return  this.baseMapper.selectOmsOrderSettingList(new OmsOrderSetting());
    }
}
