package com.chelvc.cloud.vehicle.server.service.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.chelvc.cloud.vehicle.api.constant.ReserveStatus;
import com.chelvc.cloud.vehicle.api.dto.ReservationDTO;
import com.chelvc.cloud.vehicle.api.param.ReservationModifyParam;
import com.chelvc.cloud.vehicle.api.param.ReservationQueryParam;
import com.chelvc.cloud.vehicle.server.copier.ReservationCopier;
import com.chelvc.cloud.vehicle.server.dao.ReservationMapper;
import com.chelvc.cloud.vehicle.server.entity.Reservation;
import com.chelvc.cloud.vehicle.server.service.ReservationService;
import com.chelvc.framework.base.context.SessionContextHolder;
import lombok.NonNull;
import org.apache.dubbo.config.annotation.DubboService;
import org.springframework.util.CollectionUtils;

import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 预约记录业务操作实现
 *
 * @author Woody
 * @date 2023/7/17
 */
@DubboService(interfaceClass = com.chelvc.cloud.vehicle.api.service.ReservationService.class)
public class ReservationServiceImpl extends ServiceImpl<ReservationMapper, Reservation> implements ReservationService,
        com.chelvc.cloud.vehicle.api.service.ReservationService {
    @Override
    public void addReservation(@NonNull Long merchantId, @NonNull ReservationModifyParam param) {
        Reservation reservation = ReservationCopier.INSTANCE.copying(param);
        reservation.setUserId(SessionContextHolder.getId());
        reservation.setMerchantId(merchantId);
        reservation.setStatus(ReserveStatus.PENDING);
        this.save(reservation);
    }

    @Override
    public List<ReservationDTO> listUserReservations(@NonNull ReservationQueryParam param) {
        Long userId = SessionContextHolder.getId();
        List<Reservation> reservations = this.baseMapper.listUserReservations(userId, param);
        if (CollectionUtils.isEmpty(reservations)) {
            return Collections.emptyList();
        }
        return reservations
                .stream()
                .map(o -> {
                    ReservationDTO reservationDTO = ReservationCopier.INSTANCE.copying(o);
                    reservationDTO.setMerchant(o.getMerchant());
                    return reservationDTO;
                }).collect(Collectors.toList());

    }

    @Override
    public void deleteReservation(Long id){
        this.baseMapper.deleteById(id);
    }
}
